/********************************************************************************
** Form generated from reading UI file 'lc_setsdatabasedialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_SETSDATABASEDIALOG_H
#define UI_LC_SETSDATABASEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_lcSetsDatabaseDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLineEdit *SearchEdit;
    QPushButton *SearchButton;
    QTreeWidget *SetsTree;
    QLabel *label;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcSetsDatabaseDialog)
    {
        if (lcSetsDatabaseDialog->objectName().isEmpty())
            lcSetsDatabaseDialog->setObjectName("lcSetsDatabaseDialog");
        lcSetsDatabaseDialog->resize(495, 331);
        verticalLayout = new QVBoxLayout(lcSetsDatabaseDialog);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        SearchEdit = new QLineEdit(lcSetsDatabaseDialog);
        SearchEdit->setObjectName("SearchEdit");

        horizontalLayout->addWidget(SearchEdit);

        SearchButton = new QPushButton(lcSetsDatabaseDialog);
        SearchButton->setObjectName("SearchButton");

        horizontalLayout->addWidget(SearchButton);


        verticalLayout->addLayout(horizontalLayout);

        SetsTree = new QTreeWidget(lcSetsDatabaseDialog);
        SetsTree->setObjectName("SetsTree");
        SetsTree->setRootIsDecorated(false);
        SetsTree->setUniformRowHeights(true);
        SetsTree->setAllColumnsShowFocus(true);
        SetsTree->setColumnCount(4);

        verticalLayout->addWidget(SetsTree);

        label = new QLabel(lcSetsDatabaseDialog);
        label->setObjectName("label");
        label->setOpenExternalLinks(true);

        verticalLayout->addWidget(label);

        buttonBox = new QDialogButtonBox(lcSetsDatabaseDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(lcSetsDatabaseDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcSetsDatabaseDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcSetsDatabaseDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(lcSetsDatabaseDialog);
    } // setupUi

    void retranslateUi(QDialog *lcSetsDatabaseDialog)
    {
        lcSetsDatabaseDialog->setWindowTitle(QCoreApplication::translate("lcSetsDatabaseDialog", "Sets Database", nullptr));
        SearchEdit->setPlaceholderText(QCoreApplication::translate("lcSetsDatabaseDialog", "Set Name or Number", nullptr));
        SearchButton->setText(QCoreApplication::translate("lcSetsDatabaseDialog", "Search", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = SetsTree->headerItem();
        ___qtreewidgetitem->setText(3, QCoreApplication::translate("lcSetsDatabaseDialog", "Parts", nullptr));
        ___qtreewidgetitem->setText(2, QCoreApplication::translate("lcSetsDatabaseDialog", "Year", nullptr));
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("lcSetsDatabaseDialog", "Name", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("lcSetsDatabaseDialog", "Number", nullptr));
        label->setText(QCoreApplication::translate("lcSetsDatabaseDialog", "Sets Database provided by <a href=\"https://www.rebrickable.com\">Rebrickable</a>", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcSetsDatabaseDialog: public Ui_lcSetsDatabaseDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_SETSDATABASEDIALOG_H
