/********************************************************************************
** Form generated from reading UI file 'lc_qselectdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_QSELECTDIALOG_H
#define UI_LC_QSELECTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_lcQSelectDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTreeWidget *treeWidget;
    QHBoxLayout *horizontalLayout;
    QPushButton *selectAll;
    QPushButton *selectNone;
    QPushButton *selectInvert;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcQSelectDialog)
    {
        if (lcQSelectDialog->objectName().isEmpty())
            lcQSelectDialog->setObjectName("lcQSelectDialog");
        lcQSelectDialog->resize(450, 350);
        verticalLayout = new QVBoxLayout(lcQSelectDialog);
        verticalLayout->setObjectName("verticalLayout");
        treeWidget = new QTreeWidget(lcQSelectDialog);
        QTreeWidgetItem *__qtreewidgetitem = new QTreeWidgetItem();
        __qtreewidgetitem->setText(0, QString::fromUtf8("1"));
        treeWidget->setHeaderItem(__qtreewidgetitem);
        treeWidget->setObjectName("treeWidget");
        treeWidget->header()->setVisible(false);

        verticalLayout->addWidget(treeWidget);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        selectAll = new QPushButton(lcQSelectDialog);
        selectAll->setObjectName("selectAll");

        horizontalLayout->addWidget(selectAll);

        selectNone = new QPushButton(lcQSelectDialog);
        selectNone->setObjectName("selectNone");

        horizontalLayout->addWidget(selectNone);

        selectInvert = new QPushButton(lcQSelectDialog);
        selectInvert->setObjectName("selectInvert");

        horizontalLayout->addWidget(selectInvert);


        verticalLayout->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(lcQSelectDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(lcQSelectDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcQSelectDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcQSelectDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(lcQSelectDialog);
    } // setupUi

    void retranslateUi(QDialog *lcQSelectDialog)
    {
        lcQSelectDialog->setWindowTitle(QCoreApplication::translate("lcQSelectDialog", "Select Objects", nullptr));
        selectAll->setText(QCoreApplication::translate("lcQSelectDialog", "All", nullptr));
        selectNone->setText(QCoreApplication::translate("lcQSelectDialog", "None", nullptr));
        selectInvert->setText(QCoreApplication::translate("lcQSelectDialog", "Invert", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcQSelectDialog: public Ui_lcQSelectDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_QSELECTDIALOG_H
