/****************************************************************************
** Meta object code from reading C++ file 'lc_qpreferencesdialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../qt/lc_qpreferencesdialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lc_qpreferencesdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20lcQPreferencesDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto lcQPreferencesDialog::qt_create_metaobjectdata<qt_meta_tag_ZN20lcQPreferencesDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "lcQPreferencesDialog",
        "accept",
        "",
        "on_partsLibraryBrowse_clicked",
        "on_partsArchiveBrowse_clicked",
        "on_ColorConfigBrowseButton_clicked",
        "on_MinifigSettingsBrowseButton_clicked",
        "on_povrayExecutableBrowse_clicked",
        "on_lgeoPathBrowse_clicked",
        "on_ColorTheme_currentIndexChanged",
        "Index",
        "ColorButtonClicked",
        "AutomateEdgeColor",
        "on_AutomateEdgeColor_toggled",
        "on_BlenderAddonSettingsButton_clicked",
        "on_studStyleCombo_currentIndexChanged",
        "index",
        "on_antiAliasing_toggled",
        "on_edgeLines_toggled",
        "on_ConditionalLinesCheckBox_toggled",
        "on_LineWidthSlider_valueChanged",
        "on_MeshLODSlider_valueChanged",
        "on_FadeSteps_toggled",
        "on_HighlightNewParts_toggled",
        "on_gridStuds_toggled",
        "on_gridLines_toggled",
        "on_ViewSphereSizeCombo_currentIndexChanged",
        "on_PreviewViewSphereSizeCombo_currentIndexChanged",
        "updateParts",
        "on_newCategory_clicked",
        "on_editCategory_clicked",
        "on_deleteCategory_clicked",
        "on_importCategories_clicked",
        "on_exportCategories_clicked",
        "on_resetCategories_clicked",
        "on_shortcutAssign_clicked",
        "on_shortcutRemove_clicked",
        "on_shortcutsImport_clicked",
        "on_shortcutsExport_clicked",
        "on_shortcutsReset_clicked",
        "commandChanged",
        "QTreeWidgetItem*",
        "current",
        "on_KeyboardFilterEdit_textEdited",
        "Text",
        "on_mouseAssign_clicked",
        "on_mouseRemove_clicked",
        "on_MouseImportButton_clicked",
        "on_MouseExportButton_clicked",
        "on_mouseReset_clicked",
        "MouseTreeItemChanged",
        "Current"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'accept'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_partsLibraryBrowse_clicked'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_partsArchiveBrowse_clicked'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_ColorConfigBrowseButton_clicked'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_MinifigSettingsBrowseButton_clicked'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_povrayExecutableBrowse_clicked'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_lgeoPathBrowse_clicked'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_ColorTheme_currentIndexChanged'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'ColorButtonClicked'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'AutomateEdgeColor'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_AutomateEdgeColor_toggled'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_BlenderAddonSettingsButton_clicked'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_studStyleCombo_currentIndexChanged'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'on_antiAliasing_toggled'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_edgeLines_toggled'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_ConditionalLinesCheckBox_toggled'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_LineWidthSlider_valueChanged'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_MeshLODSlider_valueChanged'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_FadeSteps_toggled'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_HighlightNewParts_toggled'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_gridStuds_toggled'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_gridLines_toggled'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_ViewSphereSizeCombo_currentIndexChanged'
        QtMocHelpers::SlotData<void(int)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'on_PreviewViewSphereSizeCombo_currentIndexChanged'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'updateParts'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_newCategory_clicked'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editCategory_clicked'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_deleteCategory_clicked'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_importCategories_clicked'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_exportCategories_clicked'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_resetCategories_clicked'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_shortcutAssign_clicked'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_shortcutRemove_clicked'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_shortcutsImport_clicked'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_shortcutsExport_clicked'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_shortcutsReset_clicked'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'commandChanged'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 42 },
        }}),
        // Slot 'on_KeyboardFilterEdit_textEdited'
        QtMocHelpers::SlotData<void(const QString &)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 44 },
        }}),
        // Slot 'on_mouseAssign_clicked'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_mouseRemove_clicked'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_MouseImportButton_clicked'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_MouseExportButton_clicked'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_mouseReset_clicked'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'MouseTreeItemChanged'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 51 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<lcQPreferencesDialog, qt_meta_tag_ZN20lcQPreferencesDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject lcQPreferencesDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20lcQPreferencesDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20lcQPreferencesDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20lcQPreferencesDialogE_t>.metaTypes,
    nullptr
} };

void lcQPreferencesDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<lcQPreferencesDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accept(); break;
        case 1: _t->on_partsLibraryBrowse_clicked(); break;
        case 2: _t->on_partsArchiveBrowse_clicked(); break;
        case 3: _t->on_ColorConfigBrowseButton_clicked(); break;
        case 4: _t->on_MinifigSettingsBrowseButton_clicked(); break;
        case 5: _t->on_povrayExecutableBrowse_clicked(); break;
        case 6: _t->on_lgeoPathBrowse_clicked(); break;
        case 7: _t->on_ColorTheme_currentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->ColorButtonClicked(); break;
        case 9: _t->AutomateEdgeColor(); break;
        case 10: _t->on_AutomateEdgeColor_toggled(); break;
        case 11: _t->on_BlenderAddonSettingsButton_clicked(); break;
        case 12: _t->on_studStyleCombo_currentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->on_antiAliasing_toggled(); break;
        case 14: _t->on_edgeLines_toggled(); break;
        case 15: _t->on_ConditionalLinesCheckBox_toggled(); break;
        case 16: _t->on_LineWidthSlider_valueChanged(); break;
        case 17: _t->on_MeshLODSlider_valueChanged(); break;
        case 18: _t->on_FadeSteps_toggled(); break;
        case 19: _t->on_HighlightNewParts_toggled(); break;
        case 20: _t->on_gridStuds_toggled(); break;
        case 21: _t->on_gridLines_toggled(); break;
        case 22: _t->on_ViewSphereSizeCombo_currentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->on_PreviewViewSphereSizeCombo_currentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->updateParts(); break;
        case 25: _t->on_newCategory_clicked(); break;
        case 26: _t->on_editCategory_clicked(); break;
        case 27: _t->on_deleteCategory_clicked(); break;
        case 28: _t->on_importCategories_clicked(); break;
        case 29: _t->on_exportCategories_clicked(); break;
        case 30: _t->on_resetCategories_clicked(); break;
        case 31: _t->on_shortcutAssign_clicked(); break;
        case 32: _t->on_shortcutRemove_clicked(); break;
        case 33: _t->on_shortcutsImport_clicked(); break;
        case 34: _t->on_shortcutsExport_clicked(); break;
        case 35: _t->on_shortcutsReset_clicked(); break;
        case 36: _t->commandChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        case 37: _t->on_KeyboardFilterEdit_textEdited((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 38: _t->on_mouseAssign_clicked(); break;
        case 39: _t->on_mouseRemove_clicked(); break;
        case 40: _t->on_MouseImportButton_clicked(); break;
        case 41: _t->on_MouseExportButton_clicked(); break;
        case 42: _t->on_mouseReset_clicked(); break;
        case 43: _t->MouseTreeItemChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *lcQPreferencesDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *lcQPreferencesDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20lcQPreferencesDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int lcQPreferencesDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 44)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 44;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 44)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 44;
    }
    return _id;
}
QT_WARNING_POP
