/********************************************************************************
** Form generated from reading UI file 'lc_pagesetupdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_PAGESETUPDIALOG_H
#define UI_LC_PAGESETUPDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_lcPageSetupDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QLineEdit *HeightEdit;
    QLineEdit *WidthEdit;
    QLabel *label_6;
    QLabel *label_5;
    QLabel *label_7;
    QComboBox *comboBox;
    QLabel *label_8;
    QComboBox *comboBox_2;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout;
    QLabel *label;
    QLineEdit *LeftEdit;
    QLabel *label_3;
    QLineEdit *RightEdit;
    QLabel *label_2;
    QLineEdit *TopEdit;
    QLabel *label_4;
    QLineEdit *BottomEdit;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcPageSetupDialog)
    {
        if (lcPageSetupDialog->objectName().isEmpty())
            lcPageSetupDialog->setObjectName("lcPageSetupDialog");
        lcPageSetupDialog->resize(400, 300);
        verticalLayout = new QVBoxLayout(lcPageSetupDialog);
        verticalLayout->setObjectName("verticalLayout");
        groupBox = new QGroupBox(lcPageSetupDialog);
        groupBox->setObjectName("groupBox");
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        HeightEdit = new QLineEdit(groupBox);
        HeightEdit->setObjectName("HeightEdit");

        gridLayout_2->addWidget(HeightEdit, 0, 3, 1, 1);

        WidthEdit = new QLineEdit(groupBox);
        WidthEdit->setObjectName("WidthEdit");

        gridLayout_2->addWidget(WidthEdit, 0, 1, 1, 1);

        label_6 = new QLabel(groupBox);
        label_6->setObjectName("label_6");

        gridLayout_2->addWidget(label_6, 0, 0, 1, 1);

        label_5 = new QLabel(groupBox);
        label_5->setObjectName("label_5");

        gridLayout_2->addWidget(label_5, 0, 2, 1, 1);

        label_7 = new QLabel(groupBox);
        label_7->setObjectName("label_7");

        gridLayout_2->addWidget(label_7, 2, 0, 1, 1);

        comboBox = new QComboBox(groupBox);
        comboBox->setObjectName("comboBox");

        gridLayout_2->addWidget(comboBox, 2, 1, 1, 3);

        label_8 = new QLabel(groupBox);
        label_8->setObjectName("label_8");

        gridLayout_2->addWidget(label_8, 1, 0, 1, 1);

        comboBox_2 = new QComboBox(groupBox);
        comboBox_2->setObjectName("comboBox_2");

        gridLayout_2->addWidget(comboBox_2, 1, 1, 1, 3);


        verticalLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(lcPageSetupDialog);
        groupBox_2->setObjectName("groupBox_2");
        gridLayout = new QGridLayout(groupBox_2);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(groupBox_2);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        LeftEdit = new QLineEdit(groupBox_2);
        LeftEdit->setObjectName("LeftEdit");

        gridLayout->addWidget(LeftEdit, 0, 1, 1, 1);

        label_3 = new QLabel(groupBox_2);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 0, 2, 1, 1);

        RightEdit = new QLineEdit(groupBox_2);
        RightEdit->setObjectName("RightEdit");

        gridLayout->addWidget(RightEdit, 0, 3, 1, 1);

        label_2 = new QLabel(groupBox_2);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        TopEdit = new QLineEdit(groupBox_2);
        TopEdit->setObjectName("TopEdit");

        gridLayout->addWidget(TopEdit, 1, 1, 1, 1);

        label_4 = new QLabel(groupBox_2);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 1, 2, 1, 1);

        BottomEdit = new QLineEdit(groupBox_2);
        BottomEdit->setObjectName("BottomEdit");

        gridLayout->addWidget(BottomEdit, 1, 3, 1, 1);


        verticalLayout->addWidget(groupBox_2);

        buttonBox = new QDialogButtonBox(lcPageSetupDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label_6->setBuddy(WidthEdit);
        label_5->setBuddy(HeightEdit);
        label->setBuddy(LeftEdit);
        label_3->setBuddy(RightEdit);
        label_2->setBuddy(TopEdit);
        label_4->setBuddy(BottomEdit);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(WidthEdit, HeightEdit);
        QWidget::setTabOrder(HeightEdit, comboBox_2);
        QWidget::setTabOrder(comboBox_2, comboBox);
        QWidget::setTabOrder(comboBox, LeftEdit);
        QWidget::setTabOrder(LeftEdit, RightEdit);
        QWidget::setTabOrder(RightEdit, TopEdit);
        QWidget::setTabOrder(TopEdit, BottomEdit);

        retranslateUi(lcPageSetupDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcPageSetupDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcPageSetupDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(lcPageSetupDialog);
    } // setupUi

    void retranslateUi(QDialog *lcPageSetupDialog)
    {
        lcPageSetupDialog->setWindowTitle(QCoreApplication::translate("lcPageSetupDialog", "Page Setup", nullptr));
        groupBox->setTitle(QCoreApplication::translate("lcPageSetupDialog", "Size", nullptr));
        label_6->setText(QCoreApplication::translate("lcPageSetupDialog", "Width:", nullptr));
        label_5->setText(QCoreApplication::translate("lcPageSetupDialog", "Height:", nullptr));
        label_7->setText(QCoreApplication::translate("lcPageSetupDialog", "Preset:", nullptr));
        label_8->setText(QCoreApplication::translate("lcPageSetupDialog", "Units:", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("lcPageSetupDialog", "Margins", nullptr));
        label->setText(QCoreApplication::translate("lcPageSetupDialog", "Left:", nullptr));
        label_3->setText(QCoreApplication::translate("lcPageSetupDialog", "Right:", nullptr));
        label_2->setText(QCoreApplication::translate("lcPageSetupDialog", "Top:", nullptr));
        label_4->setText(QCoreApplication::translate("lcPageSetupDialog", "Bottom:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcPageSetupDialog: public Ui_lcPageSetupDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_PAGESETUPDIALOG_H
