/********************************************************************************
** Form generated from reading UI file 'lc_minifigdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_MINIFIGDIALOG_H
#define UI_LC_MINIFIGDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_lcMinifigDialog
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *widget;
    QGridLayout *MinifigLayout;
    QGroupBox *TemplateGroup;
    QHBoxLayout *horizontalLayout_2;
    QComboBox *TemplateComboBox;
    QPushButton *TemplateSaveButton;
    QPushButton *TemplateDeleteButton;
    QPushButton *TemplateImportButton;
    QPushButton *TemplateExportButton;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcMinifigDialog)
    {
        if (lcMinifigDialog->objectName().isEmpty())
            lcMinifigDialog->setObjectName("lcMinifigDialog");
        lcMinifigDialog->resize(979, 416);
        verticalLayout = new QVBoxLayout(lcMinifigDialog);
        verticalLayout->setObjectName("verticalLayout");
        widget = new QWidget(lcMinifigDialog);
        widget->setObjectName("widget");
        MinifigLayout = new QGridLayout(widget);
        MinifigLayout->setObjectName("MinifigLayout");

        verticalLayout->addWidget(widget);

        TemplateGroup = new QGroupBox(lcMinifigDialog);
        TemplateGroup->setObjectName("TemplateGroup");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(TemplateGroup->sizePolicy().hasHeightForWidth());
        TemplateGroup->setSizePolicy(sizePolicy);
        horizontalLayout_2 = new QHBoxLayout(TemplateGroup);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        TemplateComboBox = new QComboBox(TemplateGroup);
        TemplateComboBox->setObjectName("TemplateComboBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(TemplateComboBox->sizePolicy().hasHeightForWidth());
        TemplateComboBox->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(TemplateComboBox);

        TemplateSaveButton = new QPushButton(TemplateGroup);
        TemplateSaveButton->setObjectName("TemplateSaveButton");

        horizontalLayout_2->addWidget(TemplateSaveButton);

        TemplateDeleteButton = new QPushButton(TemplateGroup);
        TemplateDeleteButton->setObjectName("TemplateDeleteButton");

        horizontalLayout_2->addWidget(TemplateDeleteButton);

        TemplateImportButton = new QPushButton(TemplateGroup);
        TemplateImportButton->setObjectName("TemplateImportButton");

        horizontalLayout_2->addWidget(TemplateImportButton);

        TemplateExportButton = new QPushButton(TemplateGroup);
        TemplateExportButton->setObjectName("TemplateExportButton");

        horizontalLayout_2->addWidget(TemplateExportButton);


        verticalLayout->addWidget(TemplateGroup);

        buttonBox = new QDialogButtonBox(lcMinifigDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(TemplateComboBox, TemplateSaveButton);
        QWidget::setTabOrder(TemplateSaveButton, TemplateDeleteButton);

        retranslateUi(lcMinifigDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcMinifigDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcMinifigDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(lcMinifigDialog);
    } // setupUi

    void retranslateUi(QDialog *lcMinifigDialog)
    {
        lcMinifigDialog->setWindowTitle(QCoreApplication::translate("lcMinifigDialog", "Minifig Wizard", nullptr));
        TemplateGroup->setTitle(QCoreApplication::translate("lcMinifigDialog", "Templates", nullptr));
        TemplateSaveButton->setText(QCoreApplication::translate("lcMinifigDialog", "Save...", nullptr));
        TemplateDeleteButton->setText(QCoreApplication::translate("lcMinifigDialog", "Delete...", nullptr));
        TemplateImportButton->setText(QCoreApplication::translate("lcMinifigDialog", "Import...", nullptr));
        TemplateExportButton->setText(QCoreApplication::translate("lcMinifigDialog", "Export...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcMinifigDialog: public Ui_lcMinifigDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_MINIFIGDIALOG_H
