/********************************************************************************
** Form generated from reading UI file 'lc_groupdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_GROUPDIALOG_H
#define UI_LC_GROUPDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_lcGroupDialog
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *label;
    QLineEdit *name;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcGroupDialog)
    {
        if (lcGroupDialog->objectName().isEmpty())
            lcGroupDialog->setObjectName("lcGroupDialog");
        lcGroupDialog->resize(350, 75);
        verticalLayout = new QVBoxLayout(lcGroupDialog);
        verticalLayout->setObjectName("verticalLayout");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        label = new QLabel(lcGroupDialog);
        label->setObjectName("label");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label);

        name = new QLineEdit(lcGroupDialog);
        name->setObjectName("name");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, name);


        verticalLayout->addLayout(formLayout);

        buttonBox = new QDialogButtonBox(lcGroupDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label->setBuddy(name);
#endif // QT_CONFIG(shortcut)

        retranslateUi(lcGroupDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcGroupDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcGroupDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(lcGroupDialog);
    } // setupUi

    void retranslateUi(QDialog *lcGroupDialog)
    {
        lcGroupDialog->setWindowTitle(QCoreApplication::translate("lcGroupDialog", "Group", nullptr));
        label->setText(QCoreApplication::translate("lcGroupDialog", "Name", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcGroupDialog: public Ui_lcGroupDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_GROUPDIALOG_H
