/********************************************************************************
** Form generated from reading UI file 'lc_renderdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_RENDERDIALOG_H
#define UI_LC_RENDERDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include "lc_renderdialog.h"

QT_BEGIN_NAMESPACE

class Ui_lcRenderDialog
{
public:
    QVBoxLayout *verticalLayout;
    QVBoxLayout *MainLayout;
    QHBoxLayout *outputLayout;
    QLabel *outputLabel;
    QLineEdit *OutputEdit;
    QToolButton *OutputBrowseButton;
    lcRenderPreviewWidget *preview;
    QProgressBar *RenderProgress;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcRenderDialog)
    {
        if (lcRenderDialog->objectName().isEmpty())
            lcRenderDialog->setObjectName("lcRenderDialog");
        lcRenderDialog->resize(576, 424);
        verticalLayout = new QVBoxLayout(lcRenderDialog);
        verticalLayout->setObjectName("verticalLayout");
        MainLayout = new QVBoxLayout();
        MainLayout->setObjectName("MainLayout");
        outputLayout = new QHBoxLayout();
        outputLayout->setObjectName("outputLayout");
        outputLabel = new QLabel(lcRenderDialog);
        outputLabel->setObjectName("outputLabel");

        outputLayout->addWidget(outputLabel);

        OutputEdit = new QLineEdit(lcRenderDialog);
        OutputEdit->setObjectName("OutputEdit");

        outputLayout->addWidget(OutputEdit);

        OutputBrowseButton = new QToolButton(lcRenderDialog);
        OutputBrowseButton->setObjectName("OutputBrowseButton");

        outputLayout->addWidget(OutputBrowseButton);


        MainLayout->addLayout(outputLayout);

        preview = new lcRenderPreviewWidget(lcRenderDialog);
        preview->setObjectName("preview");

        MainLayout->addWidget(preview);

        RenderProgress = new QProgressBar(lcRenderDialog);
        RenderProgress->setObjectName("RenderProgress");
        RenderProgress->setMaximum(1);
        RenderProgress->setValue(0);

        MainLayout->addWidget(RenderProgress);


        verticalLayout->addLayout(MainLayout);

        buttonBox = new QDialogButtonBox(lcRenderDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        outputLabel->setBuddy(OutputEdit);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(OutputEdit, OutputBrowseButton);

        retranslateUi(lcRenderDialog);

        QMetaObject::connectSlotsByName(lcRenderDialog);
    } // setupUi

    void retranslateUi(QDialog *lcRenderDialog)
    {
        lcRenderDialog->setWindowTitle(QCoreApplication::translate("lcRenderDialog", "Render", nullptr));
        outputLabel->setText(QCoreApplication::translate("lcRenderDialog", "Output:", nullptr));
        OutputBrowseButton->setText(QCoreApplication::translate("lcRenderDialog", "...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcRenderDialog: public Ui_lcRenderDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_RENDERDIALOG_H
