/********************************************************************************
** Form generated from reading UI file 'lc_qimagedialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_QIMAGEDIALOG_H
#define UI_LC_QIMAGEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_lcQImageDialog
{
public:
    QVBoxLayout *verticalLayout;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label;
    QLineEdit *fileName;
    QToolButton *fileNameBrowse;
    QHBoxLayout *horizontalLayout_3;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QLabel *label_3;
    QLineEdit *width;
    QLabel *label_4;
    QLineEdit *height;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_3;
    QRadioButton *rangeAll;
    QRadioButton *rangeCurrent;
    QHBoxLayout *horizontalLayout;
    QRadioButton *rangeCustom;
    QLabel *label_2;
    QLineEdit *firstStep;
    QLabel *label_7;
    QLineEdit *lastStep;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcQImageDialog)
    {
        if (lcQImageDialog->objectName().isEmpty())
            lcQImageDialog->setObjectName("lcQImageDialog");
        lcQImageDialog->resize(545, 180);
        verticalLayout = new QVBoxLayout(lcQImageDialog);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label = new QLabel(lcQImageDialog);
        label->setObjectName("label");

        horizontalLayout_2->addWidget(label);

        fileName = new QLineEdit(lcQImageDialog);
        fileName->setObjectName("fileName");

        horizontalLayout_2->addWidget(fileName);

        fileNameBrowse = new QToolButton(lcQImageDialog);
        fileNameBrowse->setObjectName("fileNameBrowse");

        horizontalLayout_2->addWidget(fileNameBrowse);


        verticalLayout_2->addLayout(horizontalLayout_2);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        groupBox = new QGroupBox(lcQImageDialog);
        groupBox->setObjectName("groupBox");
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        label_3 = new QLabel(groupBox);
        label_3->setObjectName("label_3");

        gridLayout_2->addWidget(label_3, 0, 0, 1, 1);

        width = new QLineEdit(groupBox);
        width->setObjectName("width");
        width->setMinimumSize(QSize(75, 0));
        width->setMaximumSize(QSize(75, 16777215));

        gridLayout_2->addWidget(width, 0, 1, 1, 1);

        label_4 = new QLabel(groupBox);
        label_4->setObjectName("label_4");

        gridLayout_2->addWidget(label_4, 1, 0, 1, 1);

        height = new QLineEdit(groupBox);
        height->setObjectName("height");
        height->setMinimumSize(QSize(75, 0));
        height->setMaximumSize(QSize(75, 16777215));

        gridLayout_2->addWidget(height, 1, 1, 1, 1);


        horizontalLayout_3->addWidget(groupBox);

        groupBox_2 = new QGroupBox(lcQImageDialog);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_3 = new QVBoxLayout(groupBox_2);
        verticalLayout_3->setObjectName("verticalLayout_3");
        rangeAll = new QRadioButton(groupBox_2);
        rangeAll->setObjectName("rangeAll");

        verticalLayout_3->addWidget(rangeAll);

        rangeCurrent = new QRadioButton(groupBox_2);
        rangeCurrent->setObjectName("rangeCurrent");

        verticalLayout_3->addWidget(rangeCurrent);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        rangeCustom = new QRadioButton(groupBox_2);
        rangeCustom->setObjectName("rangeCustom");

        horizontalLayout->addWidget(rangeCustom);

        label_2 = new QLabel(groupBox_2);
        label_2->setObjectName("label_2");

        horizontalLayout->addWidget(label_2);

        firstStep = new QLineEdit(groupBox_2);
        firstStep->setObjectName("firstStep");
        firstStep->setMinimumSize(QSize(75, 0));
        firstStep->setMaximumSize(QSize(75, 16777215));

        horizontalLayout->addWidget(firstStep);

        label_7 = new QLabel(groupBox_2);
        label_7->setObjectName("label_7");

        horizontalLayout->addWidget(label_7);

        lastStep = new QLineEdit(groupBox_2);
        lastStep->setObjectName("lastStep");
        lastStep->setMinimumSize(QSize(75, 0));
        lastStep->setMaximumSize(QSize(75, 16777215));

        horizontalLayout->addWidget(lastStep);


        verticalLayout_3->addLayout(horizontalLayout);


        horizontalLayout_3->addWidget(groupBox_2);


        verticalLayout_2->addLayout(horizontalLayout_3);


        verticalLayout->addLayout(verticalLayout_2);

        buttonBox = new QDialogButtonBox(lcQImageDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label->setBuddy(fileName);
        label_3->setBuddy(width);
        label_4->setBuddy(height);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(fileName, fileNameBrowse);
        QWidget::setTabOrder(fileNameBrowse, width);
        QWidget::setTabOrder(width, height);
        QWidget::setTabOrder(height, rangeAll);
        QWidget::setTabOrder(rangeAll, rangeCurrent);
        QWidget::setTabOrder(rangeCurrent, rangeCustom);
        QWidget::setTabOrder(rangeCustom, firstStep);
        QWidget::setTabOrder(firstStep, lastStep);
        QWidget::setTabOrder(lastStep, buttonBox);

        retranslateUi(lcQImageDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcQImageDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcQImageDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(lcQImageDialog);
    } // setupUi

    void retranslateUi(QDialog *lcQImageDialog)
    {
        lcQImageDialog->setWindowTitle(QCoreApplication::translate("lcQImageDialog", "Save Image", nullptr));
        label->setText(QCoreApplication::translate("lcQImageDialog", "File name:", nullptr));
        fileNameBrowse->setText(QCoreApplication::translate("lcQImageDialog", "...", nullptr));
        groupBox->setTitle(QCoreApplication::translate("lcQImageDialog", "Dimensions", nullptr));
        label_3->setText(QCoreApplication::translate("lcQImageDialog", "Width:", nullptr));
        label_4->setText(QCoreApplication::translate("lcQImageDialog", "Height:", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("lcQImageDialog", "Step Range", nullptr));
        rangeAll->setText(QCoreApplication::translate("lcQImageDialog", "All Steps", nullptr));
        rangeCurrent->setText(QCoreApplication::translate("lcQImageDialog", "Current Step", nullptr));
        rangeCustom->setText(QCoreApplication::translate("lcQImageDialog", "Custom Range", nullptr));
        label_2->setText(QCoreApplication::translate("lcQImageDialog", "From:", nullptr));
        label_7->setText(QCoreApplication::translate("lcQImageDialog", "To:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcQImageDialog: public Ui_lcQImageDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_QIMAGEDIALOG_H
