/********************************************************************************
** Form generated from reading UI file 'lc_modellistdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_MODELLISTDIALOG_H
#define UI_LC_MODELLISTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_lcModelListDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QListWidget *ModelList;
    QVBoxLayout *verticalLayout_2;
    QPushButton *NewModel;
    QPushButton *DeleteModel;
    QPushButton *RenameModel;
    QPushButton *ExportModel;
    QPushButton *DuplicateModel;
    QPushButton *MoveUp;
    QPushButton *MoveDown;
    QCheckBox *SetActiveModel;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcModelListDialog)
    {
        if (lcModelListDialog->objectName().isEmpty())
            lcModelListDialog->setObjectName("lcModelListDialog");
        lcModelListDialog->resize(421, 348);
        verticalLayout = new QVBoxLayout(lcModelListDialog);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        ModelList = new QListWidget(lcModelListDialog);
        ModelList->setObjectName("ModelList");
        ModelList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        horizontalLayout->addWidget(ModelList);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        NewModel = new QPushButton(lcModelListDialog);
        NewModel->setObjectName("NewModel");

        verticalLayout_2->addWidget(NewModel);

        DeleteModel = new QPushButton(lcModelListDialog);
        DeleteModel->setObjectName("DeleteModel");

        verticalLayout_2->addWidget(DeleteModel);

        RenameModel = new QPushButton(lcModelListDialog);
        RenameModel->setObjectName("RenameModel");

        verticalLayout_2->addWidget(RenameModel);

        ExportModel = new QPushButton(lcModelListDialog);
        ExportModel->setObjectName("ExportModel");

        verticalLayout_2->addWidget(ExportModel);

        DuplicateModel = new QPushButton(lcModelListDialog);
        DuplicateModel->setObjectName("DuplicateModel");

        verticalLayout_2->addWidget(DuplicateModel);

        MoveUp = new QPushButton(lcModelListDialog);
        MoveUp->setObjectName("MoveUp");

        verticalLayout_2->addWidget(MoveUp);

        MoveDown = new QPushButton(lcModelListDialog);
        MoveDown->setObjectName("MoveDown");

        verticalLayout_2->addWidget(MoveDown);

        SetActiveModel = new QCheckBox(lcModelListDialog);
        SetActiveModel->setObjectName("SetActiveModel");

        verticalLayout_2->addWidget(SetActiveModel);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout_2);


        verticalLayout->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(lcModelListDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(ModelList, NewModel);
        QWidget::setTabOrder(NewModel, DeleteModel);
        QWidget::setTabOrder(DeleteModel, RenameModel);
        QWidget::setTabOrder(RenameModel, ExportModel);
        QWidget::setTabOrder(ExportModel, DuplicateModel);
        QWidget::setTabOrder(DuplicateModel, MoveUp);
        QWidget::setTabOrder(MoveUp, MoveDown);
        QWidget::setTabOrder(MoveDown, SetActiveModel);

        retranslateUi(lcModelListDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcModelListDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcModelListDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(lcModelListDialog);
    } // setupUi

    void retranslateUi(QDialog *lcModelListDialog)
    {
        lcModelListDialog->setWindowTitle(QCoreApplication::translate("lcModelListDialog", "Submodels", nullptr));
        NewModel->setText(QCoreApplication::translate("lcModelListDialog", "&New...", nullptr));
        DeleteModel->setText(QCoreApplication::translate("lcModelListDialog", "D&elete...", nullptr));
        RenameModel->setText(QCoreApplication::translate("lcModelListDialog", "&Rename...", nullptr));
        ExportModel->setText(QCoreApplication::translate("lcModelListDialog", "Export...", nullptr));
        DuplicateModel->setText(QCoreApplication::translate("lcModelListDialog", "Duplicate...", nullptr));
        MoveUp->setText(QCoreApplication::translate("lcModelListDialog", "Move &Up", nullptr));
        MoveDown->setText(QCoreApplication::translate("lcModelListDialog", "Move &Down", nullptr));
        SetActiveModel->setText(QCoreApplication::translate("lcModelListDialog", "Set Active", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcModelListDialog: public Ui_lcModelListDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_MODELLISTDIALOG_H
