/********************************************************************************
** Form generated from reading UI file 'lc_qhtmldialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_QHTMLDIALOG_H
#define UI_LC_QHTMLDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_lcQHTMLDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QLabel *label_6;
    QLineEdit *outputFolder;
    QToolButton *outputFolderBrowse;
    QCheckBox *transparentImages;
    QGroupBox *groupBox_4;
    QVBoxLayout *verticalLayout_3;
    QRadioButton *allModels;
    QRadioButton *currentModelSubmodels;
    QRadioButton *currentModelOnly;
    QGroupBox *groupBox;
    QFormLayout *formLayout_2;
    QRadioButton *singlePage;
    QRadioButton *oneStepPerPage;
    QCheckBox *indexPage;
    QLabel *label;
    QLineEdit *stepWidth;
    QLabel *label_2;
    QLineEdit *stepHeight;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_4;
    QCheckBox *partsAtTheEnd;
    QCheckBox *partsAfterEachStep;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcQHTMLDialog)
    {
        if (lcQHTMLDialog->objectName().isEmpty())
            lcQHTMLDialog->setObjectName("lcQHTMLDialog");
        lcQHTMLDialog->resize(400, 460);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(lcQHTMLDialog->sizePolicy().hasHeightForWidth());
        lcQHTMLDialog->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(lcQHTMLDialog);
        verticalLayout->setObjectName("verticalLayout");
        groupBox_3 = new QGroupBox(lcQHTMLDialog);
        groupBox_3->setObjectName("groupBox_3");
        sizePolicy.setHeightForWidth(groupBox_3->sizePolicy().hasHeightForWidth());
        groupBox_3->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(groupBox_3);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label_6 = new QLabel(groupBox_3);
        label_6->setObjectName("label_6");

        horizontalLayout->addWidget(label_6);

        outputFolder = new QLineEdit(groupBox_3);
        outputFolder->setObjectName("outputFolder");

        horizontalLayout->addWidget(outputFolder);

        outputFolderBrowse = new QToolButton(groupBox_3);
        outputFolderBrowse->setObjectName("outputFolderBrowse");

        horizontalLayout->addWidget(outputFolderBrowse);


        verticalLayout_2->addLayout(horizontalLayout);

        transparentImages = new QCheckBox(groupBox_3);
        transparentImages->setObjectName("transparentImages");

        verticalLayout_2->addWidget(transparentImages);


        verticalLayout->addWidget(groupBox_3);

        groupBox_4 = new QGroupBox(lcQHTMLDialog);
        groupBox_4->setObjectName("groupBox_4");
        verticalLayout_3 = new QVBoxLayout(groupBox_4);
        verticalLayout_3->setObjectName("verticalLayout_3");
        allModels = new QRadioButton(groupBox_4);
        allModels->setObjectName("allModels");

        verticalLayout_3->addWidget(allModels);

        currentModelSubmodels = new QRadioButton(groupBox_4);
        currentModelSubmodels->setObjectName("currentModelSubmodels");

        verticalLayout_3->addWidget(currentModelSubmodels);

        currentModelOnly = new QRadioButton(groupBox_4);
        currentModelOnly->setObjectName("currentModelOnly");

        verticalLayout_3->addWidget(currentModelOnly);


        verticalLayout->addWidget(groupBox_4);

        groupBox = new QGroupBox(lcQHTMLDialog);
        groupBox->setObjectName("groupBox");
        formLayout_2 = new QFormLayout(groupBox);
        formLayout_2->setObjectName("formLayout_2");
        singlePage = new QRadioButton(groupBox);
        singlePage->setObjectName("singlePage");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::SpanningRole, singlePage);

        oneStepPerPage = new QRadioButton(groupBox);
        oneStepPerPage->setObjectName("oneStepPerPage");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::SpanningRole, oneStepPerPage);

        indexPage = new QCheckBox(groupBox);
        indexPage->setObjectName("indexPage");

        formLayout_2->setWidget(2, QFormLayout::ItemRole::SpanningRole, indexPage);

        label = new QLabel(groupBox);
        label->setObjectName("label");

        formLayout_2->setWidget(3, QFormLayout::ItemRole::LabelRole, label);

        stepWidth = new QLineEdit(groupBox);
        stepWidth->setObjectName("stepWidth");
        stepWidth->setMaximumSize(QSize(75, 16777215));

        formLayout_2->setWidget(3, QFormLayout::ItemRole::FieldRole, stepWidth);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");

        formLayout_2->setWidget(4, QFormLayout::ItemRole::LabelRole, label_2);

        stepHeight = new QLineEdit(groupBox);
        stepHeight->setObjectName("stepHeight");
        stepHeight->setMaximumSize(QSize(75, 16777215));

        formLayout_2->setWidget(4, QFormLayout::ItemRole::FieldRole, stepHeight);


        verticalLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(lcQHTMLDialog);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_4 = new QVBoxLayout(groupBox_2);
        verticalLayout_4->setObjectName("verticalLayout_4");
        partsAtTheEnd = new QCheckBox(groupBox_2);
        partsAtTheEnd->setObjectName("partsAtTheEnd");

        verticalLayout_4->addWidget(partsAtTheEnd);

        partsAfterEachStep = new QCheckBox(groupBox_2);
        partsAfterEachStep->setObjectName("partsAfterEachStep");

        verticalLayout_4->addWidget(partsAfterEachStep);


        verticalLayout->addWidget(groupBox_2);

        buttonBox = new QDialogButtonBox(lcQHTMLDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label_6->setBuddy(outputFolder);
        label->setBuddy(stepWidth);
        label_2->setBuddy(stepHeight);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(outputFolder, outputFolderBrowse);
        QWidget::setTabOrder(outputFolderBrowse, transparentImages);
        QWidget::setTabOrder(transparentImages, allModels);
        QWidget::setTabOrder(allModels, currentModelSubmodels);
        QWidget::setTabOrder(currentModelSubmodels, currentModelOnly);
        QWidget::setTabOrder(currentModelOnly, singlePage);
        QWidget::setTabOrder(singlePage, oneStepPerPage);
        QWidget::setTabOrder(oneStepPerPage, indexPage);
        QWidget::setTabOrder(indexPage, stepWidth);
        QWidget::setTabOrder(stepWidth, stepHeight);
        QWidget::setTabOrder(stepHeight, partsAtTheEnd);
        QWidget::setTabOrder(partsAtTheEnd, partsAfterEachStep);

        retranslateUi(lcQHTMLDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcQHTMLDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcQHTMLDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(lcQHTMLDialog);
    } // setupUi

    void retranslateUi(QDialog *lcQHTMLDialog)
    {
        lcQHTMLDialog->setWindowTitle(QCoreApplication::translate("lcQHTMLDialog", "HTML Options", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("lcQHTMLDialog", "General", nullptr));
        label_6->setText(QCoreApplication::translate("lcQHTMLDialog", "Output folder:", nullptr));
        outputFolderBrowse->setText(QCoreApplication::translate("lcQHTMLDialog", "...", nullptr));
        transparentImages->setText(QCoreApplication::translate("lcQHTMLDialog", "Transparent image background", nullptr));
        groupBox_4->setTitle(QCoreApplication::translate("lcQHTMLDialog", "Models", nullptr));
        allModels->setText(QCoreApplication::translate("lcQHTMLDialog", "All Models", nullptr));
        currentModelSubmodels->setText(QCoreApplication::translate("lcQHTMLDialog", "Current Model and Submodels", nullptr));
        currentModelOnly->setText(QCoreApplication::translate("lcQHTMLDialog", "Current Model Only", nullptr));
        groupBox->setTitle(QCoreApplication::translate("lcQHTMLDialog", "Steps", nullptr));
        singlePage->setText(QCoreApplication::translate("lcQHTMLDialog", "Single Page", nullptr));
        oneStepPerPage->setText(QCoreApplication::translate("lcQHTMLDialog", "One Step per Page", nullptr));
        indexPage->setText(QCoreApplication::translate("lcQHTMLDialog", "Index page", nullptr));
        label->setText(QCoreApplication::translate("lcQHTMLDialog", "Width:", nullptr));
        label_2->setText(QCoreApplication::translate("lcQHTMLDialog", "Height:", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("lcQHTMLDialog", "Parts List", nullptr));
        partsAtTheEnd->setText(QCoreApplication::translate("lcQHTMLDialog", "At the end", nullptr));
        partsAfterEachStep->setText(QCoreApplication::translate("lcQHTMLDialog", "After each step", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcQHTMLDialog: public Ui_lcQHTMLDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_QHTMLDIALOG_H
