/********************************************************************************
** Form generated from reading UI file 'lc_categorydialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_CATEGORYDIALOG_H
#define UI_LC_CATEGORYDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_lcCategoryDialog
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *label;
    QLineEdit *name;
    QLabel *label_2;
    QLineEdit *keywords;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcCategoryDialog)
    {
        if (lcCategoryDialog->objectName().isEmpty())
            lcCategoryDialog->setObjectName("lcCategoryDialog");
        lcCategoryDialog->resize(400, 95);
        verticalLayout = new QVBoxLayout(lcCategoryDialog);
        verticalLayout->setObjectName("verticalLayout");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        label = new QLabel(lcCategoryDialog);
        label->setObjectName("label");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label);

        name = new QLineEdit(lcCategoryDialog);
        name->setObjectName("name");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, name);

        label_2 = new QLabel(lcCategoryDialog);
        label_2->setObjectName("label_2");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_2);

        keywords = new QLineEdit(lcCategoryDialog);
        keywords->setObjectName("keywords");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, keywords);


        verticalLayout->addLayout(formLayout);

        buttonBox = new QDialogButtonBox(lcCategoryDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label->setBuddy(name);
        label_2->setBuddy(keywords);
#endif // QT_CONFIG(shortcut)

        retranslateUi(lcCategoryDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcCategoryDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcCategoryDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(lcCategoryDialog);
    } // setupUi

    void retranslateUi(QDialog *lcCategoryDialog)
    {
        lcCategoryDialog->setWindowTitle(QCoreApplication::translate("lcCategoryDialog", "Category", nullptr));
        label->setText(QCoreApplication::translate("lcCategoryDialog", "Name:", nullptr));
        label_2->setText(QCoreApplication::translate("lcCategoryDialog", "Keywords:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcCategoryDialog: public Ui_lcCategoryDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_CATEGORYDIALOG_H
