/********************************************************************************
** Form generated from reading UI file 'lc_partpalettedialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_PARTPALETTEDIALOG_H
#define UI_LC_PARTPALETTEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_lcPartPaletteDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QListWidget *PaletteList;
    QVBoxLayout *verticalLayout_3;
    QPushButton *NewButton;
    QPushButton *DeleteButton;
    QPushButton *RenameButton;
    QPushButton *ImportButton;
    QPushButton *MoveUpButton;
    QPushButton *MoveDownButton;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcPartPaletteDialog)
    {
        if (lcPartPaletteDialog->objectName().isEmpty())
            lcPartPaletteDialog->setObjectName("lcPartPaletteDialog");
        lcPartPaletteDialog->resize(400, 300);
        verticalLayout_2 = new QVBoxLayout(lcPartPaletteDialog);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        PaletteList = new QListWidget(lcPartPaletteDialog);
        PaletteList->setObjectName("PaletteList");

        horizontalLayout->addWidget(PaletteList);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        NewButton = new QPushButton(lcPartPaletteDialog);
        NewButton->setObjectName("NewButton");

        verticalLayout_3->addWidget(NewButton);

        DeleteButton = new QPushButton(lcPartPaletteDialog);
        DeleteButton->setObjectName("DeleteButton");

        verticalLayout_3->addWidget(DeleteButton);

        RenameButton = new QPushButton(lcPartPaletteDialog);
        RenameButton->setObjectName("RenameButton");

        verticalLayout_3->addWidget(RenameButton);

        ImportButton = new QPushButton(lcPartPaletteDialog);
        ImportButton->setObjectName("ImportButton");

        verticalLayout_3->addWidget(ImportButton);

        MoveUpButton = new QPushButton(lcPartPaletteDialog);
        MoveUpButton->setObjectName("MoveUpButton");

        verticalLayout_3->addWidget(MoveUpButton);

        MoveDownButton = new QPushButton(lcPartPaletteDialog);
        MoveDownButton->setObjectName("MoveDownButton");

        verticalLayout_3->addWidget(MoveDownButton);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout_3);


        verticalLayout_2->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(lcPartPaletteDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);

        QWidget::setTabOrder(PaletteList, NewButton);
        QWidget::setTabOrder(NewButton, DeleteButton);
        QWidget::setTabOrder(DeleteButton, RenameButton);
        QWidget::setTabOrder(RenameButton, ImportButton);
        QWidget::setTabOrder(ImportButton, MoveUpButton);
        QWidget::setTabOrder(MoveUpButton, MoveDownButton);

        retranslateUi(lcPartPaletteDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcPartPaletteDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcPartPaletteDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(lcPartPaletteDialog);
    } // setupUi

    void retranslateUi(QDialog *lcPartPaletteDialog)
    {
        lcPartPaletteDialog->setWindowTitle(QCoreApplication::translate("lcPartPaletteDialog", "Part Palettes", nullptr));
        NewButton->setText(QCoreApplication::translate("lcPartPaletteDialog", "New...", nullptr));
        DeleteButton->setText(QCoreApplication::translate("lcPartPaletteDialog", "Delete...", nullptr));
        RenameButton->setText(QCoreApplication::translate("lcPartPaletteDialog", "Rename...", nullptr));
        ImportButton->setText(QCoreApplication::translate("lcPartPaletteDialog", "Import...", nullptr));
        MoveUpButton->setText(QCoreApplication::translate("lcPartPaletteDialog", "Move Up", nullptr));
        MoveDownButton->setText(QCoreApplication::translate("lcPartPaletteDialog", "Move Down", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcPartPaletteDialog: public Ui_lcPartPaletteDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_PARTPALETTEDIALOG_H
