/****************************************************************************
** Meta object code from reading C++ file 'lc_partselectionwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../common/lc_partselectionwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lc_partselectionwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN27lcPartSelectionItemDelegateE_t {};
} // unnamed namespace

template <> constexpr inline auto lcPartSelectionItemDelegate::qt_create_metaobjectdata<qt_meta_tag_ZN27lcPartSelectionItemDelegateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "lcPartSelectionItemDelegate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<lcPartSelectionItemDelegate, qt_meta_tag_ZN27lcPartSelectionItemDelegateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject lcPartSelectionItemDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<QStyledItemDelegate::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27lcPartSelectionItemDelegateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27lcPartSelectionItemDelegateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27lcPartSelectionItemDelegateE_t>.metaTypes,
    nullptr
} };

void lcPartSelectionItemDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<lcPartSelectionItemDelegate *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *lcPartSelectionItemDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *lcPartSelectionItemDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27lcPartSelectionItemDelegateE_t>.strings))
        return static_cast<void*>(this);
    return QStyledItemDelegate::qt_metacast(_clname);
}

int lcPartSelectionItemDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStyledItemDelegate::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN24lcPartSelectionListModelE_t {};
} // unnamed namespace

template <> constexpr inline auto lcPartSelectionListModel::qt_create_metaobjectdata<qt_meta_tag_ZN24lcPartSelectionListModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "lcPartSelectionListModel",
        "ThumbnailReady",
        "",
        "lcPartThumbnailId",
        "ThumbnailId",
        "QPixmap",
        "Pixmap"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'ThumbnailReady'
        QtMocHelpers::SlotData<void(lcPartThumbnailId, QPixmap)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<lcPartSelectionListModel, qt_meta_tag_ZN24lcPartSelectionListModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject lcPartSelectionListModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24lcPartSelectionListModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24lcPartSelectionListModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24lcPartSelectionListModelE_t>.metaTypes,
    nullptr
} };

void lcPartSelectionListModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<lcPartSelectionListModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ThumbnailReady((*reinterpret_cast<std::add_pointer_t<lcPartThumbnailId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *lcPartSelectionListModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *lcPartSelectionListModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24lcPartSelectionListModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int lcPartSelectionListModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN23lcPartSelectionListViewE_t {};
} // unnamed namespace

template <> constexpr inline auto lcPartSelectionListView::qt_create_metaobjectdata<qt_meta_tag_ZN23lcPartSelectionListViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "lcPartSelectionListView",
        "PartPicked",
        "",
        "PieceInfo*",
        "Info",
        "CustomContextMenuRequested",
        "QPoint",
        "Pos",
        "SetNoIcons",
        "SetSmallIcons",
        "SetMediumIcons",
        "SetLargeIcons",
        "SetExtraLargeIcons",
        "ToggleRegularExpressionFilter",
        "TogglePartNames",
        "ToggleDecoratedParts",
        "TogglePartAliases",
        "ToggleCaseSensitiveFilter",
        "ToggleFileNameFilter",
        "TogglePartDescriptionFilter",
        "ToggleListMode",
        "ToggleFixedColor",
        "DoubleClicked",
        "QModelIndex",
        "Index"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'PartPicked'
        QtMocHelpers::SignalData<void(PieceInfo *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'CustomContextMenuRequested'
        QtMocHelpers::SlotData<void(QPoint)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'SetNoIcons'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetSmallIcons'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetMediumIcons'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetLargeIcons'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'SetExtraLargeIcons'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ToggleRegularExpressionFilter'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'TogglePartNames'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ToggleDecoratedParts'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'TogglePartAliases'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ToggleCaseSensitiveFilter'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ToggleFileNameFilter'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'TogglePartDescriptionFilter'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ToggleListMode'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ToggleFixedColor'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'DoubleClicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<lcPartSelectionListView, qt_meta_tag_ZN23lcPartSelectionListViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject lcPartSelectionListView::staticMetaObject = { {
    QMetaObject::SuperData::link<QListView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23lcPartSelectionListViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23lcPartSelectionListViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23lcPartSelectionListViewE_t>.metaTypes,
    nullptr
} };

void lcPartSelectionListView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<lcPartSelectionListView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->PartPicked((*reinterpret_cast<std::add_pointer_t<PieceInfo*>>(_a[1]))); break;
        case 1: _t->CustomContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 2: _t->SetNoIcons(); break;
        case 3: _t->SetSmallIcons(); break;
        case 4: _t->SetMediumIcons(); break;
        case 5: _t->SetLargeIcons(); break;
        case 6: _t->SetExtraLargeIcons(); break;
        case 7: _t->ToggleRegularExpressionFilter(); break;
        case 8: _t->TogglePartNames(); break;
        case 9: _t->ToggleDecoratedParts(); break;
        case 10: _t->TogglePartAliases(); break;
        case 11: _t->ToggleCaseSensitiveFilter(); break;
        case 12: _t->ToggleFileNameFilter(); break;
        case 13: _t->TogglePartDescriptionFilter(); break;
        case 14: _t->ToggleListMode(); break;
        case 15: _t->ToggleFixedColor(); break;
        case 16: _t->DoubleClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (lcPartSelectionListView::*)(PieceInfo * )>(_a, &lcPartSelectionListView::PartPicked, 0))
            return;
    }
}

const QMetaObject *lcPartSelectionListView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *lcPartSelectionListView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23lcPartSelectionListViewE_t>.strings))
        return static_cast<void*>(this);
    return QListView::qt_metacast(_clname);
}

int lcPartSelectionListView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void lcPartSelectionListView::PartPicked(PieceInfo * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN21lcPartSelectionWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto lcPartSelectionWidget::qt_create_metaobjectdata<qt_meta_tag_ZN21lcPartSelectionWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "lcPartSelectionWidget",
        "PartPicked",
        "",
        "PieceInfo*",
        "Info",
        "CurrentPartChanged",
        "AddToPalette",
        "RemoveFromPalette",
        "DockLocationChanged",
        "Qt::DockWidgetArea",
        "Area",
        "FilterChanged",
        "Text",
        "FilterCategoriesChanged",
        "FilterTriggered",
        "FilterCaseTriggered",
        "FilterCategoriesTriggered",
        "CategoryChanged",
        "QTreeWidgetItem*",
        "Current",
        "Previous",
        "PartViewSelectionChanged",
        "QModelIndex",
        "PartViewPartPicked",
        "OptionsMenuAboutToShow",
        "EditPartPalettes"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'PartPicked'
        QtMocHelpers::SignalData<void(PieceInfo *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'CurrentPartChanged'
        QtMocHelpers::SignalData<void(PieceInfo *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'AddToPalette'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'RemoveFromPalette'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'DockLocationChanged'
        QtMocHelpers::SlotData<void(Qt::DockWidgetArea)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'FilterChanged'
        QtMocHelpers::SlotData<void(const QString &)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'FilterCategoriesChanged'
        QtMocHelpers::SlotData<void(const QString &)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'FilterTriggered'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'FilterCaseTriggered'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'FilterCategoriesTriggered'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'CategoryChanged'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, QTreeWidgetItem *)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 18, 20 },
        }}),
        // Slot 'PartViewSelectionChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 22, 19 }, { 0x80000000 | 22, 20 },
        }}),
        // Slot 'PartViewPartPicked'
        QtMocHelpers::SlotData<void(PieceInfo *)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'OptionsMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'EditPartPalettes'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<lcPartSelectionWidget, qt_meta_tag_ZN21lcPartSelectionWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject lcPartSelectionWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21lcPartSelectionWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21lcPartSelectionWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21lcPartSelectionWidgetE_t>.metaTypes,
    nullptr
} };

void lcPartSelectionWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<lcPartSelectionWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->PartPicked((*reinterpret_cast<std::add_pointer_t<PieceInfo*>>(_a[1]))); break;
        case 1: _t->CurrentPartChanged((*reinterpret_cast<std::add_pointer_t<PieceInfo*>>(_a[1]))); break;
        case 2: _t->AddToPalette(); break;
        case 3: _t->RemoveFromPalette(); break;
        case 4: _t->DockLocationChanged((*reinterpret_cast<std::add_pointer_t<Qt::DockWidgetArea>>(_a[1]))); break;
        case 5: _t->FilterChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->FilterCategoriesChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->FilterTriggered(); break;
        case 8: _t->FilterCaseTriggered(); break;
        case 9: _t->FilterCategoriesTriggered(); break;
        case 10: _t->CategoryChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[2]))); break;
        case 11: _t->PartViewSelectionChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 12: _t->PartViewPartPicked((*reinterpret_cast<std::add_pointer_t<PieceInfo*>>(_a[1]))); break;
        case 13: _t->OptionsMenuAboutToShow(); break;
        case 14: _t->EditPartPalettes(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (lcPartSelectionWidget::*)(PieceInfo * )>(_a, &lcPartSelectionWidget::PartPicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (lcPartSelectionWidget::*)(PieceInfo * )>(_a, &lcPartSelectionWidget::CurrentPartChanged, 1))
            return;
    }
}

const QMetaObject *lcPartSelectionWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *lcPartSelectionWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21lcPartSelectionWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int lcPartSelectionWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void lcPartSelectionWidget::PartPicked(PieceInfo * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void lcPartSelectionWidget::CurrentPartChanged(PieceInfo * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
