/********************************************************************************
** Form generated from reading UI file 'lc_qpreferencesdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_QPREFERENCESDIALOG_H
#define UI_LC_QPREFERENCESDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_lcQPreferencesDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tabGeneral;
    QFormLayout *formLayout;
    QGroupBox *ApplicationGroup;
    QGridLayout *gridLayout_4;
    QLabel *label_19;
    QComboBox *Language;
    QComboBox *checkForUpdates;
    QLabel *label_10;
    QSpacerItem *horizontalSpacer_8;
    QLabel *label_25;
    QComboBox *ColorTheme;
    QLabel *label_4;
    QLineEdit *authorName;
    QCheckBox *fixedDirectionKeys;
    QCheckBox *autoLoadMostRecent;
    QGroupBox *CustomPathsGroup;
    QFormLayout *formLayout_2;
    QLabel *label_5;
    QHBoxLayout *horizontalLayout_3;
    QLineEdit *partsLibrary;
    QToolButton *partsLibraryBrowse;
    QToolButton *partsArchiveBrowse;
    QLabel *label_17;
    QHBoxLayout *ColorConfigLayout;
    QLineEdit *ColorConfigEdit;
    QToolButton *ColorConfigBrowseButton;
    QLabel *label_18;
    QHBoxLayout *MinifigSettingsLayout;
    QLineEdit *MinifigSettingsEdit;
    QToolButton *MinifigSettingsBrowseButton;
    QLabel *povrayLabel;
    QHBoxLayout *povrayLayout;
    QLineEdit *povrayExecutable;
    QToolButton *povrayExecutableBrowse;
    QLabel *label_9;
    QHBoxLayout *horizontalLayout_10;
    QLineEdit *lgeoPath;
    QToolButton *lgeoPathBrowse;
    QCheckBox *RestoreTabLayout;
    QWidget *tabRendering;
    QVBoxLayout *verticalLayout_8;
    QWidget *widget;
    QGridLayout *gridLayout;
    QCheckBox *MeshLOD;
    QToolButton *FadeStepsColor;
    QComboBox *ShadingMode;
    QCheckBox *FadeSteps;
    QSlider *MeshLODSlider;
    QLabel *MeshLODLabel;
    QCheckBox *antiAliasing;
    QLabel *label_24;
    QCheckBox *HighlightNewParts;
    QLabel *label_3;
    QComboBox *studStyleCombo;
    QToolButton *HighContrastButton;
    QSpacerItem *horizontalSpacer_2;
    QComboBox *antiAliasingSamples;
    QToolButton *HighlightNewPartsColor;
    QLabel *label_39;
    QToolButton *BlenderAddonSettingsButton;
    QGroupBox *groupBox_8;
    QGridLayout *gridLayout_5;
    QLabel *LineWidthLabel;
    QCheckBox *ConditionalLinesCheckBox;
    QSpacerItem *horizontalSpacer_9;
    QLabel *label_26;
    QSlider *LineWidthSlider;
    QCheckBox *edgeLines;
    QCheckBox *AutomateEdgeColor;
    QToolButton *AutomateEdgeColorButton;
    QSpacerItem *verticalSpacer;
    QWidget *tab;
    QVBoxLayout *verticalLayout_13;
    QGroupBox *groupBox_6;
    QGridLayout *gridLayout_3;
    QCheckBox *AxisIconCheckBox;
    QLabel *ViewSphereLocationLabel;
    QLabel *label_21;
    QComboBox *ViewSphereSizeCombo;
    QComboBox *ViewSphereLocationCombo;
    QSpacerItem *horizontalSpacer_7;
    QLabel *AxisIconLocationLabel;
    QComboBox *AxisIconLocationCombo;
    QGroupBox *groupBox_4;
    QGridLayout *gridLayout_2;
    QLabel *label;
    QSpacerItem *horizontalSpacer_5;
    QCheckBox *gridLines;
    QLineEdit *gridLineSpacing;
    QCheckBox *GridOriginCheckBox;
    QCheckBox *gridStuds;
    QGroupBox *PreviewGroup;
    QGridLayout *gridLayout_6;
    QCheckBox *PreviewAxisIconCheckBox;
    QComboBox *PreviewViewSphereLocationCombo;
    QLabel *PreviewViewSphereLocationLabel;
    QComboBox *PreviewViewSphereSizeCombo;
    QLabel *label_8;
    QSpacerItem *horizontalSpacer_11;
    QSpacerItem *verticalSpacer_2;
    QWidget *ColorTab;
    QGridLayout *gridLayout_9;
    QGroupBox *groupBox_11;
    QGridLayout *gridLayout_8;
    QLabel *label_15;
    QToolButton *MarqueeBorderColorButton;
    QLabel *label_32;
    QToolButton *MarqueeFillColorButton;
    QLabel *label_14;
    QToolButton *AxesColorButton;
    QLabel *label_16;
    QToolButton *InactiveViewColorButton;
    QLabel *label_2;
    QLabel *label_31;
    QToolButton *TextColorButton;
    QToolButton *OverlayColorButton;
    QToolButton *ActiveViewColorButton;
    QSpacerItem *horizontalSpacer_16;
    QLabel *label_6;
    QSpacerItem *horizontalSpacer_18;
    QGroupBox *groupBox_9;
    QHBoxLayout *horizontalLayout_16;
    QLabel *label_22;
    QToolButton *ViewSphereColorButton;
    QSpacerItem *horizontalSpacer_21;
    QLabel *label_23;
    QToolButton *ViewSphereTextColorButton;
    QSpacerItem *horizontalSpacer_22;
    QLabel *label_20;
    QToolButton *ViewSphereHighlightColorButton;
    QSpacerItem *horizontalSpacer_14;
    QSpacerItem *verticalSpacer_3;
    QGroupBox *groupBox_10;
    QHBoxLayout *horizontalLayout_17;
    QLabel *label_29;
    QToolButton *gridLineColor;
    QSpacerItem *horizontalSpacer_20;
    QLabel *label_30;
    QToolButton *gridStudColor;
    QSpacerItem *horizontalSpacer_13;
    QGroupBox *groupBox_7;
    QGridLayout *gridLayout_7;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_27;
    QToolButton *BackgroundGradient1ColorButton;
    QSpacerItem *horizontalSpacer_19;
    QLabel *label_28;
    QToolButton *BackgroundGradient2ColorButton;
    QSpacerItem *horizontalSpacer_10;
    QRadioButton *BackgroundSolidRadio;
    QRadioButton *BackgroundGradientRadio;
    QHBoxLayout *horizontalLayout_11;
    QToolButton *BackgroundSolidColorButton;
    QSpacerItem *horizontalSpacer_12;
    QGroupBox *groupBox_12;
    QGridLayout *gridLayout_10;
    QLabel *label_37;
    QToolButton *LightColorButton;
    QSpacerItem *horizontalSpacer_15;
    QLabel *label_38;
    QToolButton *CameraColorButton;
    QLabel *label_36;
    QToolButton *ControlPointColorButton;
    QToolButton *ControlPointFocusedColorButton;
    QToolButton *ObjectSelectedColorButton;
    QToolButton *ObjectFocusedColorButton;
    QLabel *label_35;
    QLabel *label_33;
    QLabel *label_34;
    QSpacerItem *horizontalSpacer_17;
    QWidget *tabCategories;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_7;
    QSplitter *splitter;
    QTreeWidget *categoriesTree;
    QTreeWidget *partsTree;
    QHBoxLayout *horizontalLayout;
    QPushButton *importCategories;
    QPushButton *exportCategories;
    QPushButton *resetCategories;
    QSpacerItem *horizontalSpacer_4;
    QGroupBox *groupBox_2;
    QHBoxLayout *horizontalLayout_7;
    QPushButton *newCategory;
    QPushButton *editCategory;
    QPushButton *deleteCategory;
    QSpacerItem *horizontalSpacer;
    QWidget *tabKeyboard;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_4;
    QVBoxLayout *verticalLayout_5;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_6;
    QTreeWidget *commandList;
    QLineEdit *KeyboardFilterEdit;
    QHBoxLayout *horizontalLayout_9;
    QPushButton *shortcutsImport;
    QPushButton *shortcutsExport;
    QPushButton *shortcutsReset;
    QSpacerItem *horizontalSpacer_3;
    QHBoxLayout *horizontalLayout_6;
    QGroupBox *shortcutGroup;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_7;
    QLineEdit *shortcutEdit;
    QPushButton *shortcutAssign;
    QPushButton *shortcutRemove;
    QVBoxLayout *verticalLayout_4;
    QWidget *tabMouse;
    QVBoxLayout *verticalLayout_10;
    QVBoxLayout *verticalLayout_9;
    QGroupBox *groupBox_5;
    QVBoxLayout *verticalLayout_11;
    QVBoxLayout *verticalLayout_12;
    QTreeWidget *mouseTree;
    QHBoxLayout *horizontalLayout_13;
    QPushButton *MouseImportButton;
    QPushButton *MouseExportButton;
    QPushButton *mouseReset;
    QSpacerItem *horizontalSpacer_6;
    QGroupBox *MouseShortcutGroup;
    QHBoxLayout *horizontalLayout_12;
    QHBoxLayout *horizontalLayout_14;
    QLabel *label_11;
    QComboBox *mouseButton;
    QLabel *label_12;
    QCheckBox *mouseControl;
    QCheckBox *mouseAlt;
    QCheckBox *mouseShift;
    QPushButton *mouseAssign;
    QPushButton *mouseRemove;
    QHBoxLayout *horizontalLayout_15;
    QLabel *label_13;
    QSlider *mouseSensitivity;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcQPreferencesDialog)
    {
        if (lcQPreferencesDialog->objectName().isEmpty())
            lcQPreferencesDialog->setObjectName("lcQPreferencesDialog");
        lcQPreferencesDialog->resize(621, 503);
        verticalLayout = new QVBoxLayout(lcQPreferencesDialog);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(lcQPreferencesDialog);
        tabWidget->setObjectName("tabWidget");
        tabGeneral = new QWidget();
        tabGeneral->setObjectName("tabGeneral");
        formLayout = new QFormLayout(tabGeneral);
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        ApplicationGroup = new QGroupBox(tabGeneral);
        ApplicationGroup->setObjectName("ApplicationGroup");
        gridLayout_4 = new QGridLayout(ApplicationGroup);
        gridLayout_4->setObjectName("gridLayout_4");
        label_19 = new QLabel(ApplicationGroup);
        label_19->setObjectName("label_19");

        gridLayout_4->addWidget(label_19, 1, 0, 1, 1);

        Language = new QComboBox(ApplicationGroup);
        Language->addItem(QString());
        Language->addItem(QString());
        Language->addItem(QString());
        Language->addItem(QString());
        Language->addItem(QString());
        Language->addItem(QString());
        Language->addItem(QString());
        Language->setObjectName("Language");

        gridLayout_4->addWidget(Language, 1, 1, 1, 1);

        checkForUpdates = new QComboBox(ApplicationGroup);
        checkForUpdates->addItem(QString());
        checkForUpdates->addItem(QString());
        checkForUpdates->addItem(QString());
        checkForUpdates->setObjectName("checkForUpdates");

        gridLayout_4->addWidget(checkForUpdates, 2, 1, 1, 1);

        label_10 = new QLabel(ApplicationGroup);
        label_10->setObjectName("label_10");

        gridLayout_4->addWidget(label_10, 2, 0, 1, 1);

        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer_8, 1, 2, 1, 1);

        label_25 = new QLabel(ApplicationGroup);
        label_25->setObjectName("label_25");

        gridLayout_4->addWidget(label_25, 0, 0, 1, 1);

        ColorTheme = new QComboBox(ApplicationGroup);
        ColorTheme->addItem(QString());
        ColorTheme->addItem(QString());
        ColorTheme->setObjectName("ColorTheme");

        gridLayout_4->addWidget(ColorTheme, 0, 1, 1, 1);


        formLayout->setWidget(1, QFormLayout::ItemRole::SpanningRole, ApplicationGroup);

        label_4 = new QLabel(tabGeneral);
        label_4->setObjectName("label_4");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, label_4);

        authorName = new QLineEdit(tabGeneral);
        authorName->setObjectName("authorName");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, authorName);

        fixedDirectionKeys = new QCheckBox(tabGeneral);
        fixedDirectionKeys->setObjectName("fixedDirectionKeys");

        formLayout->setWidget(9, QFormLayout::ItemRole::SpanningRole, fixedDirectionKeys);

        autoLoadMostRecent = new QCheckBox(tabGeneral);
        autoLoadMostRecent->setObjectName("autoLoadMostRecent");

        formLayout->setWidget(10, QFormLayout::ItemRole::SpanningRole, autoLoadMostRecent);

        CustomPathsGroup = new QGroupBox(tabGeneral);
        CustomPathsGroup->setObjectName("CustomPathsGroup");
        formLayout_2 = new QFormLayout(CustomPathsGroup);
        formLayout_2->setObjectName("formLayout_2");
        label_5 = new QLabel(CustomPathsGroup);
        label_5->setObjectName("label_5");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, label_5);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        partsLibrary = new QLineEdit(CustomPathsGroup);
        partsLibrary->setObjectName("partsLibrary");

        horizontalLayout_3->addWidget(partsLibrary);

        partsLibraryBrowse = new QToolButton(CustomPathsGroup);
        partsLibraryBrowse->setObjectName("partsLibraryBrowse");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/resources/file_open.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        partsLibraryBrowse->setIcon(icon);

        horizontalLayout_3->addWidget(partsLibraryBrowse);

        partsArchiveBrowse = new QToolButton(CustomPathsGroup);
        partsArchiveBrowse->setObjectName("partsArchiveBrowse");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/resources/archive.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        partsArchiveBrowse->setIcon(icon1);

        horizontalLayout_3->addWidget(partsArchiveBrowse);


        formLayout_2->setLayout(0, QFormLayout::ItemRole::FieldRole, horizontalLayout_3);

        label_17 = new QLabel(CustomPathsGroup);
        label_17->setObjectName("label_17");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, label_17);

        ColorConfigLayout = new QHBoxLayout();
        ColorConfigLayout->setObjectName("ColorConfigLayout");
        ColorConfigEdit = new QLineEdit(CustomPathsGroup);
        ColorConfigEdit->setObjectName("ColorConfigEdit");

        ColorConfigLayout->addWidget(ColorConfigEdit);

        ColorConfigBrowseButton = new QToolButton(CustomPathsGroup);
        ColorConfigBrowseButton->setObjectName("ColorConfigBrowseButton");

        ColorConfigLayout->addWidget(ColorConfigBrowseButton);


        formLayout_2->setLayout(1, QFormLayout::ItemRole::FieldRole, ColorConfigLayout);

        label_18 = new QLabel(CustomPathsGroup);
        label_18->setObjectName("label_18");

        formLayout_2->setWidget(2, QFormLayout::ItemRole::LabelRole, label_18);

        MinifigSettingsLayout = new QHBoxLayout();
        MinifigSettingsLayout->setObjectName("MinifigSettingsLayout");
        MinifigSettingsEdit = new QLineEdit(CustomPathsGroup);
        MinifigSettingsEdit->setObjectName("MinifigSettingsEdit");

        MinifigSettingsLayout->addWidget(MinifigSettingsEdit);

        MinifigSettingsBrowseButton = new QToolButton(CustomPathsGroup);
        MinifigSettingsBrowseButton->setObjectName("MinifigSettingsBrowseButton");

        MinifigSettingsLayout->addWidget(MinifigSettingsBrowseButton);


        formLayout_2->setLayout(2, QFormLayout::ItemRole::FieldRole, MinifigSettingsLayout);

        povrayLabel = new QLabel(CustomPathsGroup);
        povrayLabel->setObjectName("povrayLabel");

        formLayout_2->setWidget(3, QFormLayout::ItemRole::LabelRole, povrayLabel);

        povrayLayout = new QHBoxLayout();
        povrayLayout->setObjectName("povrayLayout");
        povrayExecutable = new QLineEdit(CustomPathsGroup);
        povrayExecutable->setObjectName("povrayExecutable");

        povrayLayout->addWidget(povrayExecutable);

        povrayExecutableBrowse = new QToolButton(CustomPathsGroup);
        povrayExecutableBrowse->setObjectName("povrayExecutableBrowse");

        povrayLayout->addWidget(povrayExecutableBrowse);


        formLayout_2->setLayout(3, QFormLayout::ItemRole::FieldRole, povrayLayout);

        label_9 = new QLabel(CustomPathsGroup);
        label_9->setObjectName("label_9");

        formLayout_2->setWidget(4, QFormLayout::ItemRole::LabelRole, label_9);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        lgeoPath = new QLineEdit(CustomPathsGroup);
        lgeoPath->setObjectName("lgeoPath");

        horizontalLayout_10->addWidget(lgeoPath);

        lgeoPathBrowse = new QToolButton(CustomPathsGroup);
        lgeoPathBrowse->setObjectName("lgeoPathBrowse");

        horizontalLayout_10->addWidget(lgeoPathBrowse);


        formLayout_2->setLayout(4, QFormLayout::ItemRole::FieldRole, horizontalLayout_10);


        formLayout->setWidget(0, QFormLayout::ItemRole::SpanningRole, CustomPathsGroup);

        RestoreTabLayout = new QCheckBox(tabGeneral);
        RestoreTabLayout->setObjectName("RestoreTabLayout");

        formLayout->setWidget(11, QFormLayout::ItemRole::SpanningRole, RestoreTabLayout);

        tabWidget->addTab(tabGeneral, QString());
        tabRendering = new QWidget();
        tabRendering->setObjectName("tabRendering");
        verticalLayout_8 = new QVBoxLayout(tabRendering);
        verticalLayout_8->setObjectName("verticalLayout_8");
        widget = new QWidget(tabRendering);
        widget->setObjectName("widget");
        gridLayout = new QGridLayout(widget);
        gridLayout->setObjectName("gridLayout");
        MeshLOD = new QCheckBox(widget);
        MeshLOD->setObjectName("MeshLOD");

        gridLayout->addWidget(MeshLOD, 12, 0, 1, 1);

        FadeStepsColor = new QToolButton(widget);
        FadeStepsColor->setObjectName("FadeStepsColor");

        gridLayout->addWidget(FadeStepsColor, 10, 1, 1, 1);

        ShadingMode = new QComboBox(widget);
        ShadingMode->addItem(QString());
        ShadingMode->addItem(QString());
        ShadingMode->addItem(QString());
        ShadingMode->setObjectName("ShadingMode");

        gridLayout->addWidget(ShadingMode, 14, 1, 1, 1);

        FadeSteps = new QCheckBox(widget);
        FadeSteps->setObjectName("FadeSteps");

        gridLayout->addWidget(FadeSteps, 10, 0, 1, 1);

        MeshLODSlider = new QSlider(widget);
        MeshLODSlider->setObjectName("MeshLODSlider");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(MeshLODSlider->sizePolicy().hasHeightForWidth());
        MeshLODSlider->setSizePolicy(sizePolicy);
        MeshLODSlider->setOrientation(Qt::Horizontal);
        MeshLODSlider->setTickPosition(QSlider::NoTicks);

        gridLayout->addWidget(MeshLODSlider, 12, 1, 1, 1);

        MeshLODLabel = new QLabel(widget);
        MeshLODLabel->setObjectName("MeshLODLabel");

        gridLayout->addWidget(MeshLODLabel, 12, 2, 1, 1);

        antiAliasing = new QCheckBox(widget);
        antiAliasing->setObjectName("antiAliasing");

        gridLayout->addWidget(antiAliasing, 0, 0, 1, 1);

        label_24 = new QLabel(widget);
        label_24->setObjectName("label_24");

        gridLayout->addWidget(label_24, 13, 0, 1, 1);

        HighlightNewParts = new QCheckBox(widget);
        HighlightNewParts->setObjectName("HighlightNewParts");

        gridLayout->addWidget(HighlightNewParts, 11, 0, 1, 1);

        label_3 = new QLabel(widget);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 14, 0, 1, 1);

        studStyleCombo = new QComboBox(widget);
        studStyleCombo->addItem(QString());
        studStyleCombo->addItem(QString());
        studStyleCombo->addItem(QString());
        studStyleCombo->addItem(QString());
        studStyleCombo->addItem(QString());
        studStyleCombo->addItem(QString());
        studStyleCombo->addItem(QString());
        studStyleCombo->addItem(QString());
        studStyleCombo->setObjectName("studStyleCombo");

        gridLayout->addWidget(studStyleCombo, 13, 1, 1, 1);

        HighContrastButton = new QToolButton(widget);
        HighContrastButton->setObjectName("HighContrastButton");

        gridLayout->addWidget(HighContrastButton, 13, 2, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_2, 0, 3, 1, 1);

        antiAliasingSamples = new QComboBox(widget);
        antiAliasingSamples->addItem(QString());
        antiAliasingSamples->addItem(QString());
        antiAliasingSamples->addItem(QString());
        antiAliasingSamples->setObjectName("antiAliasingSamples");

        gridLayout->addWidget(antiAliasingSamples, 0, 1, 1, 1);

        HighlightNewPartsColor = new QToolButton(widget);
        HighlightNewPartsColor->setObjectName("HighlightNewPartsColor");

        gridLayout->addWidget(HighlightNewPartsColor, 11, 1, 1, 1);

        label_39 = new QLabel(widget);
        label_39->setObjectName("label_39");

        gridLayout->addWidget(label_39, 15, 0, 1, 1);

        BlenderAddonSettingsButton = new QToolButton(widget);
        BlenderAddonSettingsButton->setObjectName("BlenderAddonSettingsButton");

        gridLayout->addWidget(BlenderAddonSettingsButton, 15, 1, 1, 1);


        verticalLayout_8->addWidget(widget);

        groupBox_8 = new QGroupBox(tabRendering);
        groupBox_8->setObjectName("groupBox_8");
        gridLayout_5 = new QGridLayout(groupBox_8);
        gridLayout_5->setObjectName("gridLayout_5");
        LineWidthLabel = new QLabel(groupBox_8);
        LineWidthLabel->setObjectName("LineWidthLabel");

        gridLayout_5->addWidget(LineWidthLabel, 3, 2, 1, 1);

        ConditionalLinesCheckBox = new QCheckBox(groupBox_8);
        ConditionalLinesCheckBox->setObjectName("ConditionalLinesCheckBox");

        gridLayout_5->addWidget(ConditionalLinesCheckBox, 1, 0, 1, 1);

        horizontalSpacer_9 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_5->addItem(horizontalSpacer_9, 3, 3, 1, 1);

        label_26 = new QLabel(groupBox_8);
        label_26->setObjectName("label_26");

        gridLayout_5->addWidget(label_26, 3, 0, 1, 1);

        LineWidthSlider = new QSlider(groupBox_8);
        LineWidthSlider->setObjectName("LineWidthSlider");
        sizePolicy.setHeightForWidth(LineWidthSlider->sizePolicy().hasHeightForWidth());
        LineWidthSlider->setSizePolicy(sizePolicy);
        LineWidthSlider->setOrientation(Qt::Horizontal);
        LineWidthSlider->setTickPosition(QSlider::NoTicks);

        gridLayout_5->addWidget(LineWidthSlider, 3, 1, 1, 1);

        edgeLines = new QCheckBox(groupBox_8);
        edgeLines->setObjectName("edgeLines");

        gridLayout_5->addWidget(edgeLines, 0, 0, 1, 1);

        AutomateEdgeColor = new QCheckBox(groupBox_8);
        AutomateEdgeColor->setObjectName("AutomateEdgeColor");

        gridLayout_5->addWidget(AutomateEdgeColor, 2, 0, 1, 1);

        AutomateEdgeColorButton = new QToolButton(groupBox_8);
        AutomateEdgeColorButton->setObjectName("AutomateEdgeColorButton");

        gridLayout_5->addWidget(AutomateEdgeColorButton, 2, 1, 1, 1);


        verticalLayout_8->addWidget(groupBox_8);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_8->addItem(verticalSpacer);

        tabWidget->addTab(tabRendering, QString());
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_13 = new QVBoxLayout(tab);
        verticalLayout_13->setObjectName("verticalLayout_13");
        groupBox_6 = new QGroupBox(tab);
        groupBox_6->setObjectName("groupBox_6");
        gridLayout_3 = new QGridLayout(groupBox_6);
        gridLayout_3->setObjectName("gridLayout_3");
        AxisIconCheckBox = new QCheckBox(groupBox_6);
        AxisIconCheckBox->setObjectName("AxisIconCheckBox");

        gridLayout_3->addWidget(AxisIconCheckBox, 1, 0, 1, 1);

        ViewSphereLocationLabel = new QLabel(groupBox_6);
        ViewSphereLocationLabel->setObjectName("ViewSphereLocationLabel");

        gridLayout_3->addWidget(ViewSphereLocationLabel, 0, 2, 1, 1);

        label_21 = new QLabel(groupBox_6);
        label_21->setObjectName("label_21");

        gridLayout_3->addWidget(label_21, 0, 0, 1, 1);

        ViewSphereSizeCombo = new QComboBox(groupBox_6);
        ViewSphereSizeCombo->addItem(QString());
        ViewSphereSizeCombo->addItem(QString());
        ViewSphereSizeCombo->addItem(QString());
        ViewSphereSizeCombo->addItem(QString());
        ViewSphereSizeCombo->setObjectName("ViewSphereSizeCombo");

        gridLayout_3->addWidget(ViewSphereSizeCombo, 0, 1, 1, 1);

        ViewSphereLocationCombo = new QComboBox(groupBox_6);
        ViewSphereLocationCombo->addItem(QString());
        ViewSphereLocationCombo->addItem(QString());
        ViewSphereLocationCombo->addItem(QString());
        ViewSphereLocationCombo->addItem(QString());
        ViewSphereLocationCombo->setObjectName("ViewSphereLocationCombo");

        gridLayout_3->addWidget(ViewSphereLocationCombo, 0, 3, 1, 1);

        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_7, 0, 4, 1, 1);

        AxisIconLocationLabel = new QLabel(groupBox_6);
        AxisIconLocationLabel->setObjectName("AxisIconLocationLabel");

        gridLayout_3->addWidget(AxisIconLocationLabel, 1, 2, 1, 1);

        AxisIconLocationCombo = new QComboBox(groupBox_6);
        AxisIconLocationCombo->addItem(QString());
        AxisIconLocationCombo->addItem(QString());
        AxisIconLocationCombo->addItem(QString());
        AxisIconLocationCombo->addItem(QString());
        AxisIconLocationCombo->setObjectName("AxisIconLocationCombo");

        gridLayout_3->addWidget(AxisIconLocationCombo, 1, 3, 1, 1);


        verticalLayout_13->addWidget(groupBox_6);

        groupBox_4 = new QGroupBox(tab);
        groupBox_4->setObjectName("groupBox_4");
        gridLayout_2 = new QGridLayout(groupBox_4);
        gridLayout_2->setObjectName("gridLayout_2");
        label = new QLabel(groupBox_4);
        label->setObjectName("label");

        gridLayout_2->addWidget(label, 1, 2, 1, 1);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_5, 1, 3, 1, 1);

        gridLines = new QCheckBox(groupBox_4);
        gridLines->setObjectName("gridLines");

        gridLayout_2->addWidget(gridLines, 1, 0, 1, 1);

        gridLineSpacing = new QLineEdit(groupBox_4);
        gridLineSpacing->setObjectName("gridLineSpacing");
        gridLineSpacing->setMaximumSize(QSize(75, 16777215));

        gridLayout_2->addWidget(gridLineSpacing, 1, 1, 1, 1);

        GridOriginCheckBox = new QCheckBox(groupBox_4);
        GridOriginCheckBox->setObjectName("GridOriginCheckBox");

        gridLayout_2->addWidget(GridOriginCheckBox, 2, 0, 1, 3);

        gridStuds = new QCheckBox(groupBox_4);
        gridStuds->setObjectName("gridStuds");

        gridLayout_2->addWidget(gridStuds, 0, 0, 1, 3);


        verticalLayout_13->addWidget(groupBox_4);

        PreviewGroup = new QGroupBox(tab);
        PreviewGroup->setObjectName("PreviewGroup");
        gridLayout_6 = new QGridLayout(PreviewGroup);
        gridLayout_6->setObjectName("gridLayout_6");
        PreviewAxisIconCheckBox = new QCheckBox(PreviewGroup);
        PreviewAxisIconCheckBox->setObjectName("PreviewAxisIconCheckBox");

        gridLayout_6->addWidget(PreviewAxisIconCheckBox, 2, 0, 1, 2);

        PreviewViewSphereLocationCombo = new QComboBox(PreviewGroup);
        PreviewViewSphereLocationCombo->addItem(QString());
        PreviewViewSphereLocationCombo->addItem(QString());
        PreviewViewSphereLocationCombo->addItem(QString());
        PreviewViewSphereLocationCombo->addItem(QString());
        PreviewViewSphereLocationCombo->setObjectName("PreviewViewSphereLocationCombo");

        gridLayout_6->addWidget(PreviewViewSphereLocationCombo, 1, 3, 1, 1);

        PreviewViewSphereLocationLabel = new QLabel(PreviewGroup);
        PreviewViewSphereLocationLabel->setObjectName("PreviewViewSphereLocationLabel");

        gridLayout_6->addWidget(PreviewViewSphereLocationLabel, 1, 2, 1, 1);

        PreviewViewSphereSizeCombo = new QComboBox(PreviewGroup);
        PreviewViewSphereSizeCombo->addItem(QString());
        PreviewViewSphereSizeCombo->addItem(QString());
        PreviewViewSphereSizeCombo->addItem(QString());
        PreviewViewSphereSizeCombo->addItem(QString());
        PreviewViewSphereSizeCombo->setObjectName("PreviewViewSphereSizeCombo");

        gridLayout_6->addWidget(PreviewViewSphereSizeCombo, 1, 1, 1, 1);

        label_8 = new QLabel(PreviewGroup);
        label_8->setObjectName("label_8");

        gridLayout_6->addWidget(label_8, 1, 0, 1, 1);

        horizontalSpacer_11 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_6->addItem(horizontalSpacer_11, 1, 4, 1, 1);


        verticalLayout_13->addWidget(PreviewGroup);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_13->addItem(verticalSpacer_2);

        tabWidget->addTab(tab, QString());
        ColorTab = new QWidget();
        ColorTab->setObjectName("ColorTab");
        gridLayout_9 = new QGridLayout(ColorTab);
        gridLayout_9->setObjectName("gridLayout_9");
        groupBox_11 = new QGroupBox(ColorTab);
        groupBox_11->setObjectName("groupBox_11");
        gridLayout_8 = new QGridLayout(groupBox_11);
        gridLayout_8->setObjectName("gridLayout_8");
        label_15 = new QLabel(groupBox_11);
        label_15->setObjectName("label_15");

        gridLayout_8->addWidget(label_15, 3, 0, 1, 1);

        MarqueeBorderColorButton = new QToolButton(groupBox_11);
        MarqueeBorderColorButton->setObjectName("MarqueeBorderColorButton");

        gridLayout_8->addWidget(MarqueeBorderColorButton, 3, 1, 1, 1);

        label_32 = new QLabel(groupBox_11);
        label_32->setObjectName("label_32");

        gridLayout_8->addWidget(label_32, 0, 3, 1, 1);

        MarqueeFillColorButton = new QToolButton(groupBox_11);
        MarqueeFillColorButton->setObjectName("MarqueeFillColorButton");

        gridLayout_8->addWidget(MarqueeFillColorButton, 3, 4, 1, 1);

        label_14 = new QLabel(groupBox_11);
        label_14->setObjectName("label_14");

        gridLayout_8->addWidget(label_14, 2, 0, 1, 1);

        AxesColorButton = new QToolButton(groupBox_11);
        AxesColorButton->setObjectName("AxesColorButton");

        gridLayout_8->addWidget(AxesColorButton, 2, 4, 1, 1);

        label_16 = new QLabel(groupBox_11);
        label_16->setObjectName("label_16");

        gridLayout_8->addWidget(label_16, 3, 3, 1, 1);

        InactiveViewColorButton = new QToolButton(groupBox_11);
        InactiveViewColorButton->setObjectName("InactiveViewColorButton");

        gridLayout_8->addWidget(InactiveViewColorButton, 0, 4, 1, 1);

        label_2 = new QLabel(groupBox_11);
        label_2->setObjectName("label_2");

        gridLayout_8->addWidget(label_2, 5, 0, 1, 1);

        label_31 = new QLabel(groupBox_11);
        label_31->setObjectName("label_31");

        gridLayout_8->addWidget(label_31, 2, 3, 1, 1);

        TextColorButton = new QToolButton(groupBox_11);
        TextColorButton->setObjectName("TextColorButton");

        gridLayout_8->addWidget(TextColorButton, 2, 1, 1, 1);

        OverlayColorButton = new QToolButton(groupBox_11);
        OverlayColorButton->setObjectName("OverlayColorButton");

        gridLayout_8->addWidget(OverlayColorButton, 5, 1, 1, 1);

        ActiveViewColorButton = new QToolButton(groupBox_11);
        ActiveViewColorButton->setObjectName("ActiveViewColorButton");

        gridLayout_8->addWidget(ActiveViewColorButton, 0, 1, 1, 1);

        horizontalSpacer_16 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_8->addItem(horizontalSpacer_16, 0, 5, 1, 1);

        label_6 = new QLabel(groupBox_11);
        label_6->setObjectName("label_6");

        gridLayout_8->addWidget(label_6, 0, 0, 1, 1);

        horizontalSpacer_18 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout_8->addItem(horizontalSpacer_18, 0, 2, 1, 1);


        gridLayout_9->addWidget(groupBox_11, 2, 0, 1, 2);

        groupBox_9 = new QGroupBox(ColorTab);
        groupBox_9->setObjectName("groupBox_9");
        horizontalLayout_16 = new QHBoxLayout(groupBox_9);
        horizontalLayout_16->setObjectName("horizontalLayout_16");
        label_22 = new QLabel(groupBox_9);
        label_22->setObjectName("label_22");

        horizontalLayout_16->addWidget(label_22);

        ViewSphereColorButton = new QToolButton(groupBox_9);
        ViewSphereColorButton->setObjectName("ViewSphereColorButton");

        horizontalLayout_16->addWidget(ViewSphereColorButton);

        horizontalSpacer_21 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_16->addItem(horizontalSpacer_21);

        label_23 = new QLabel(groupBox_9);
        label_23->setObjectName("label_23");

        horizontalLayout_16->addWidget(label_23);

        ViewSphereTextColorButton = new QToolButton(groupBox_9);
        ViewSphereTextColorButton->setObjectName("ViewSphereTextColorButton");

        horizontalLayout_16->addWidget(ViewSphereTextColorButton);

        horizontalSpacer_22 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_16->addItem(horizontalSpacer_22);

        label_20 = new QLabel(groupBox_9);
        label_20->setObjectName("label_20");

        horizontalLayout_16->addWidget(label_20);

        ViewSphereHighlightColorButton = new QToolButton(groupBox_9);
        ViewSphereHighlightColorButton->setObjectName("ViewSphereHighlightColorButton");

        horizontalLayout_16->addWidget(ViewSphereHighlightColorButton);

        horizontalSpacer_14 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_16->addItem(horizontalSpacer_14);


        gridLayout_9->addWidget(groupBox_9, 4, 1, 1, 1);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_9->addItem(verticalSpacer_3, 6, 0, 1, 1);

        groupBox_10 = new QGroupBox(ColorTab);
        groupBox_10->setObjectName("groupBox_10");
        horizontalLayout_17 = new QHBoxLayout(groupBox_10);
        horizontalLayout_17->setObjectName("horizontalLayout_17");
        label_29 = new QLabel(groupBox_10);
        label_29->setObjectName("label_29");

        horizontalLayout_17->addWidget(label_29);

        gridLineColor = new QToolButton(groupBox_10);
        gridLineColor->setObjectName("gridLineColor");

        horizontalLayout_17->addWidget(gridLineColor);

        horizontalSpacer_20 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_17->addItem(horizontalSpacer_20);

        label_30 = new QLabel(groupBox_10);
        label_30->setObjectName("label_30");

        horizontalLayout_17->addWidget(label_30);

        gridStudColor = new QToolButton(groupBox_10);
        gridStudColor->setObjectName("gridStudColor");

        horizontalLayout_17->addWidget(gridStudColor);

        horizontalSpacer_13 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_17->addItem(horizontalSpacer_13);


        gridLayout_9->addWidget(groupBox_10, 4, 0, 1, 1);

        groupBox_7 = new QGroupBox(ColorTab);
        groupBox_7->setObjectName("groupBox_7");
        gridLayout_7 = new QGridLayout(groupBox_7);
        gridLayout_7->setObjectName("gridLayout_7");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_27 = new QLabel(groupBox_7);
        label_27->setObjectName("label_27");

        horizontalLayout_2->addWidget(label_27);

        BackgroundGradient1ColorButton = new QToolButton(groupBox_7);
        BackgroundGradient1ColorButton->setObjectName("BackgroundGradient1ColorButton");

        horizontalLayout_2->addWidget(BackgroundGradient1ColorButton);

        horizontalSpacer_19 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_19);

        label_28 = new QLabel(groupBox_7);
        label_28->setObjectName("label_28");

        horizontalLayout_2->addWidget(label_28);

        BackgroundGradient2ColorButton = new QToolButton(groupBox_7);
        BackgroundGradient2ColorButton->setObjectName("BackgroundGradient2ColorButton");

        horizontalLayout_2->addWidget(BackgroundGradient2ColorButton);

        horizontalSpacer_10 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_10);


        gridLayout_7->addLayout(horizontalLayout_2, 1, 1, 1, 1);

        BackgroundSolidRadio = new QRadioButton(groupBox_7);
        BackgroundSolidRadio->setObjectName("BackgroundSolidRadio");

        gridLayout_7->addWidget(BackgroundSolidRadio, 0, 0, 1, 1);

        BackgroundGradientRadio = new QRadioButton(groupBox_7);
        BackgroundGradientRadio->setObjectName("BackgroundGradientRadio");

        gridLayout_7->addWidget(BackgroundGradientRadio, 1, 0, 1, 1);

        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName("horizontalLayout_11");
        BackgroundSolidColorButton = new QToolButton(groupBox_7);
        BackgroundSolidColorButton->setObjectName("BackgroundSolidColorButton");

        horizontalLayout_11->addWidget(BackgroundSolidColorButton);

        horizontalSpacer_12 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_12);


        gridLayout_7->addLayout(horizontalLayout_11, 0, 1, 1, 1);


        gridLayout_9->addWidget(groupBox_7, 0, 0, 1, 2);

        groupBox_12 = new QGroupBox(ColorTab);
        groupBox_12->setObjectName("groupBox_12");
        gridLayout_10 = new QGridLayout(groupBox_12);
        gridLayout_10->setObjectName("gridLayout_10");
        label_37 = new QLabel(groupBox_12);
        label_37->setObjectName("label_37");

        gridLayout_10->addWidget(label_37, 1, 3, 1, 1);

        LightColorButton = new QToolButton(groupBox_12);
        LightColorButton->setObjectName("LightColorButton");

        gridLayout_10->addWidget(LightColorButton, 1, 4, 1, 1);

        horizontalSpacer_15 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout_10->addItem(horizontalSpacer_15, 0, 2, 1, 1);

        label_38 = new QLabel(groupBox_12);
        label_38->setObjectName("label_38");

        gridLayout_10->addWidget(label_38, 0, 3, 1, 1);

        CameraColorButton = new QToolButton(groupBox_12);
        CameraColorButton->setObjectName("CameraColorButton");

        gridLayout_10->addWidget(CameraColorButton, 1, 1, 1, 1);

        label_36 = new QLabel(groupBox_12);
        label_36->setObjectName("label_36");

        gridLayout_10->addWidget(label_36, 2, 3, 1, 1);

        ControlPointColorButton = new QToolButton(groupBox_12);
        ControlPointColorButton->setObjectName("ControlPointColorButton");

        gridLayout_10->addWidget(ControlPointColorButton, 2, 1, 1, 1);

        ControlPointFocusedColorButton = new QToolButton(groupBox_12);
        ControlPointFocusedColorButton->setObjectName("ControlPointFocusedColorButton");

        gridLayout_10->addWidget(ControlPointFocusedColorButton, 2, 4, 1, 1);

        ObjectSelectedColorButton = new QToolButton(groupBox_12);
        ObjectSelectedColorButton->setObjectName("ObjectSelectedColorButton");

        gridLayout_10->addWidget(ObjectSelectedColorButton, 0, 1, 1, 1);

        ObjectFocusedColorButton = new QToolButton(groupBox_12);
        ObjectFocusedColorButton->setObjectName("ObjectFocusedColorButton");

        gridLayout_10->addWidget(ObjectFocusedColorButton, 0, 4, 1, 1);

        label_35 = new QLabel(groupBox_12);
        label_35->setObjectName("label_35");

        gridLayout_10->addWidget(label_35, 2, 0, 1, 1);

        label_33 = new QLabel(groupBox_12);
        label_33->setObjectName("label_33");

        gridLayout_10->addWidget(label_33, 0, 0, 1, 1);

        label_34 = new QLabel(groupBox_12);
        label_34->setObjectName("label_34");

        gridLayout_10->addWidget(label_34, 1, 0, 1, 1);

        horizontalSpacer_17 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_10->addItem(horizontalSpacer_17, 0, 5, 1, 1);


        gridLayout_9->addWidget(groupBox_12, 1, 0, 1, 2);

        tabWidget->addTab(ColorTab, QString());
        tabCategories = new QWidget();
        tabCategories->setObjectName("tabCategories");
        verticalLayout_3 = new QVBoxLayout(tabCategories);
        verticalLayout_3->setObjectName("verticalLayout_3");
        groupBox_3 = new QGroupBox(tabCategories);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_7 = new QVBoxLayout(groupBox_3);
        verticalLayout_7->setObjectName("verticalLayout_7");
        splitter = new QSplitter(groupBox_3);
        splitter->setObjectName("splitter");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(splitter->sizePolicy().hasHeightForWidth());
        splitter->setSizePolicy(sizePolicy1);
        splitter->setOrientation(Qt::Horizontal);
        categoriesTree = new QTreeWidget(splitter);
        categoriesTree->setObjectName("categoriesTree");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(categoriesTree->sizePolicy().hasHeightForWidth());
        categoriesTree->setSizePolicy(sizePolicy2);
        categoriesTree->setRootIsDecorated(false);
        categoriesTree->setUniformRowHeights(true);
        categoriesTree->setItemsExpandable(false);
        categoriesTree->setExpandsOnDoubleClick(false);
        splitter->addWidget(categoriesTree);
        partsTree = new QTreeWidget(splitter);
        partsTree->setObjectName("partsTree");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy3.setHorizontalStretch(2);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(partsTree->sizePolicy().hasHeightForWidth());
        partsTree->setSizePolicy(sizePolicy3);
        partsTree->setSelectionMode(QAbstractItemView::NoSelection);
        partsTree->setRootIsDecorated(false);
        partsTree->setUniformRowHeights(true);
        partsTree->setItemsExpandable(false);
        partsTree->setSortingEnabled(true);
        partsTree->setColumnCount(2);
        splitter->addWidget(partsTree);
        partsTree->header()->setProperty("showSortIndicator", QVariant(true));

        verticalLayout_7->addWidget(splitter);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        importCategories = new QPushButton(groupBox_3);
        importCategories->setObjectName("importCategories");

        horizontalLayout->addWidget(importCategories);

        exportCategories = new QPushButton(groupBox_3);
        exportCategories->setObjectName("exportCategories");

        horizontalLayout->addWidget(exportCategories);

        resetCategories = new QPushButton(groupBox_3);
        resetCategories->setObjectName("resetCategories");

        horizontalLayout->addWidget(resetCategories);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_4);


        verticalLayout_7->addLayout(horizontalLayout);


        verticalLayout_3->addWidget(groupBox_3);

        groupBox_2 = new QGroupBox(tabCategories);
        groupBox_2->setObjectName("groupBox_2");
        horizontalLayout_7 = new QHBoxLayout(groupBox_2);
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        newCategory = new QPushButton(groupBox_2);
        newCategory->setObjectName("newCategory");

        horizontalLayout_7->addWidget(newCategory);

        editCategory = new QPushButton(groupBox_2);
        editCategory->setObjectName("editCategory");

        horizontalLayout_7->addWidget(editCategory);

        deleteCategory = new QPushButton(groupBox_2);
        deleteCategory->setObjectName("deleteCategory");

        horizontalLayout_7->addWidget(deleteCategory);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer);


        verticalLayout_3->addWidget(groupBox_2);

        tabWidget->addTab(tabCategories, QString());
        tabKeyboard = new QWidget();
        tabKeyboard->setObjectName("tabKeyboard");
        verticalLayout_2 = new QVBoxLayout(tabKeyboard);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName("verticalLayout_5");
        groupBox = new QGroupBox(tabKeyboard);
        groupBox->setObjectName("groupBox");
        verticalLayout_6 = new QVBoxLayout(groupBox);
        verticalLayout_6->setObjectName("verticalLayout_6");
        commandList = new QTreeWidget(groupBox);
        commandList->setObjectName("commandList");
        commandList->setRootIsDecorated(true);
        commandList->setUniformRowHeights(true);

        verticalLayout_6->addWidget(commandList);

        KeyboardFilterEdit = new QLineEdit(groupBox);
        KeyboardFilterEdit->setObjectName("KeyboardFilterEdit");

        verticalLayout_6->addWidget(KeyboardFilterEdit);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        shortcutsImport = new QPushButton(groupBox);
        shortcutsImport->setObjectName("shortcutsImport");

        horizontalLayout_9->addWidget(shortcutsImport);

        shortcutsExport = new QPushButton(groupBox);
        shortcutsExport->setObjectName("shortcutsExport");

        horizontalLayout_9->addWidget(shortcutsExport);

        shortcutsReset = new QPushButton(groupBox);
        shortcutsReset->setObjectName("shortcutsReset");

        horizontalLayout_9->addWidget(shortcutsReset);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_3);


        verticalLayout_6->addLayout(horizontalLayout_9);


        verticalLayout_5->addWidget(groupBox);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");

        verticalLayout_5->addLayout(horizontalLayout_6);

        shortcutGroup = new QGroupBox(tabKeyboard);
        shortcutGroup->setObjectName("shortcutGroup");
        horizontalLayout_5 = new QHBoxLayout(shortcutGroup);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label_7 = new QLabel(shortcutGroup);
        label_7->setObjectName("label_7");

        horizontalLayout_5->addWidget(label_7);

        shortcutEdit = new QLineEdit(shortcutGroup);
        shortcutEdit->setObjectName("shortcutEdit");

        horizontalLayout_5->addWidget(shortcutEdit);

        shortcutAssign = new QPushButton(shortcutGroup);
        shortcutAssign->setObjectName("shortcutAssign");

        horizontalLayout_5->addWidget(shortcutAssign);

        shortcutRemove = new QPushButton(shortcutGroup);
        shortcutRemove->setObjectName("shortcutRemove");

        horizontalLayout_5->addWidget(shortcutRemove);

        shortcutEdit->raise();
        label_7->raise();
        shortcutAssign->raise();
        shortcutRemove->raise();

        verticalLayout_5->addWidget(shortcutGroup);


        horizontalLayout_4->addLayout(verticalLayout_5);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");

        horizontalLayout_4->addLayout(verticalLayout_4);


        verticalLayout_2->addLayout(horizontalLayout_4);

        tabWidget->addTab(tabKeyboard, QString());
        tabMouse = new QWidget();
        tabMouse->setObjectName("tabMouse");
        verticalLayout_10 = new QVBoxLayout(tabMouse);
        verticalLayout_10->setObjectName("verticalLayout_10");
        verticalLayout_9 = new QVBoxLayout();
        verticalLayout_9->setObjectName("verticalLayout_9");
        groupBox_5 = new QGroupBox(tabMouse);
        groupBox_5->setObjectName("groupBox_5");
        verticalLayout_11 = new QVBoxLayout(groupBox_5);
        verticalLayout_11->setObjectName("verticalLayout_11");
        verticalLayout_12 = new QVBoxLayout();
        verticalLayout_12->setObjectName("verticalLayout_12");
        mouseTree = new QTreeWidget(groupBox_5);
        mouseTree->setObjectName("mouseTree");
        mouseTree->setRootIsDecorated(false);
        mouseTree->setUniformRowHeights(true);

        verticalLayout_12->addWidget(mouseTree);

        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setObjectName("horizontalLayout_13");
        MouseImportButton = new QPushButton(groupBox_5);
        MouseImportButton->setObjectName("MouseImportButton");

        horizontalLayout_13->addWidget(MouseImportButton);

        MouseExportButton = new QPushButton(groupBox_5);
        MouseExportButton->setObjectName("MouseExportButton");

        horizontalLayout_13->addWidget(MouseExportButton);

        mouseReset = new QPushButton(groupBox_5);
        mouseReset->setObjectName("mouseReset");

        horizontalLayout_13->addWidget(mouseReset);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_6);


        verticalLayout_12->addLayout(horizontalLayout_13);


        verticalLayout_11->addLayout(verticalLayout_12);


        verticalLayout_9->addWidget(groupBox_5);


        verticalLayout_10->addLayout(verticalLayout_9);

        MouseShortcutGroup = new QGroupBox(tabMouse);
        MouseShortcutGroup->setObjectName("MouseShortcutGroup");
        horizontalLayout_12 = new QHBoxLayout(MouseShortcutGroup);
        horizontalLayout_12->setObjectName("horizontalLayout_12");
        horizontalLayout_14 = new QHBoxLayout();
        horizontalLayout_14->setObjectName("horizontalLayout_14");
        label_11 = new QLabel(MouseShortcutGroup);
        label_11->setObjectName("label_11");

        horizontalLayout_14->addWidget(label_11);

        mouseButton = new QComboBox(MouseShortcutGroup);
        mouseButton->addItem(QString());
        mouseButton->addItem(QString());
        mouseButton->addItem(QString());
        mouseButton->addItem(QString());
        mouseButton->setObjectName("mouseButton");

        horizontalLayout_14->addWidget(mouseButton);

        label_12 = new QLabel(MouseShortcutGroup);
        label_12->setObjectName("label_12");

        horizontalLayout_14->addWidget(label_12);

        mouseControl = new QCheckBox(MouseShortcutGroup);
        mouseControl->setObjectName("mouseControl");

        horizontalLayout_14->addWidget(mouseControl);

        mouseAlt = new QCheckBox(MouseShortcutGroup);
        mouseAlt->setObjectName("mouseAlt");

        horizontalLayout_14->addWidget(mouseAlt);

        mouseShift = new QCheckBox(MouseShortcutGroup);
        mouseShift->setObjectName("mouseShift");

        horizontalLayout_14->addWidget(mouseShift);

        mouseAssign = new QPushButton(MouseShortcutGroup);
        mouseAssign->setObjectName("mouseAssign");

        horizontalLayout_14->addWidget(mouseAssign);

        mouseRemove = new QPushButton(MouseShortcutGroup);
        mouseRemove->setObjectName("mouseRemove");

        horizontalLayout_14->addWidget(mouseRemove);


        horizontalLayout_12->addLayout(horizontalLayout_14);


        verticalLayout_10->addWidget(MouseShortcutGroup);

        horizontalLayout_15 = new QHBoxLayout();
        horizontalLayout_15->setObjectName("horizontalLayout_15");
        label_13 = new QLabel(tabMouse);
        label_13->setObjectName("label_13");

        horizontalLayout_15->addWidget(label_13);

        mouseSensitivity = new QSlider(tabMouse);
        mouseSensitivity->setObjectName("mouseSensitivity");
        mouseSensitivity->setMinimum(1);
        mouseSensitivity->setMaximum(20);
        mouseSensitivity->setPageStep(5);
        mouseSensitivity->setOrientation(Qt::Horizontal);

        horizontalLayout_15->addWidget(mouseSensitivity);


        verticalLayout_10->addLayout(horizontalLayout_15);

        tabWidget->addTab(tabMouse, QString());

        verticalLayout->addWidget(tabWidget);

        buttonBox = new QDialogButtonBox(lcQPreferencesDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label_19->setBuddy(Language);
        label_10->setBuddy(checkForUpdates);
        label_25->setBuddy(Language);
        label_4->setBuddy(authorName);
        label_5->setBuddy(partsLibrary);
        label_17->setBuddy(ColorConfigEdit);
        label_18->setBuddy(MinifigSettingsEdit);
        povrayLabel->setBuddy(povrayExecutable);
        label_9->setBuddy(lgeoPath);
        label_7->setBuddy(shortcutEdit);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(tabWidget, partsLibrary);
        QWidget::setTabOrder(partsLibrary, partsLibraryBrowse);
        QWidget::setTabOrder(partsLibraryBrowse, partsArchiveBrowse);
        QWidget::setTabOrder(partsArchiveBrowse, ColorConfigEdit);
        QWidget::setTabOrder(ColorConfigEdit, ColorConfigBrowseButton);
        QWidget::setTabOrder(ColorConfigBrowseButton, MinifigSettingsEdit);
        QWidget::setTabOrder(MinifigSettingsEdit, MinifigSettingsBrowseButton);
        QWidget::setTabOrder(MinifigSettingsBrowseButton, povrayExecutable);
        QWidget::setTabOrder(povrayExecutable, povrayExecutableBrowse);
        QWidget::setTabOrder(povrayExecutableBrowse, lgeoPath);
        QWidget::setTabOrder(lgeoPath, lgeoPathBrowse);
        QWidget::setTabOrder(lgeoPathBrowse, ColorTheme);
        QWidget::setTabOrder(ColorTheme, Language);
        QWidget::setTabOrder(Language, checkForUpdates);
        QWidget::setTabOrder(checkForUpdates, authorName);
        QWidget::setTabOrder(authorName, fixedDirectionKeys);
        QWidget::setTabOrder(fixedDirectionKeys, autoLoadMostRecent);
        QWidget::setTabOrder(autoLoadMostRecent, RestoreTabLayout);
        QWidget::setTabOrder(RestoreTabLayout, antiAliasing);
        QWidget::setTabOrder(antiAliasing, antiAliasingSamples);
        QWidget::setTabOrder(antiAliasingSamples, FadeSteps);
        QWidget::setTabOrder(FadeSteps, FadeStepsColor);
        QWidget::setTabOrder(FadeStepsColor, HighlightNewParts);
        QWidget::setTabOrder(HighlightNewParts, HighlightNewPartsColor);
        QWidget::setTabOrder(HighlightNewPartsColor, MeshLOD);
        QWidget::setTabOrder(MeshLOD, MeshLODSlider);
        QWidget::setTabOrder(MeshLODSlider, studStyleCombo);
        QWidget::setTabOrder(studStyleCombo, HighContrastButton);
        QWidget::setTabOrder(HighContrastButton, ShadingMode);
        QWidget::setTabOrder(ShadingMode, BlenderAddonSettingsButton);
        QWidget::setTabOrder(BlenderAddonSettingsButton, edgeLines);
        QWidget::setTabOrder(edgeLines, ConditionalLinesCheckBox);
        QWidget::setTabOrder(ConditionalLinesCheckBox, AutomateEdgeColor);
        QWidget::setTabOrder(AutomateEdgeColor, AutomateEdgeColorButton);
        QWidget::setTabOrder(AutomateEdgeColorButton, LineWidthSlider);
        QWidget::setTabOrder(LineWidthSlider, ViewSphereSizeCombo);
        QWidget::setTabOrder(ViewSphereSizeCombo, ViewSphereLocationCombo);
        QWidget::setTabOrder(ViewSphereLocationCombo, AxisIconCheckBox);
        QWidget::setTabOrder(AxisIconCheckBox, AxisIconLocationCombo);
        QWidget::setTabOrder(AxisIconLocationCombo, gridStuds);
        QWidget::setTabOrder(gridStuds, gridLines);
        QWidget::setTabOrder(gridLines, gridLineSpacing);
        QWidget::setTabOrder(gridLineSpacing, GridOriginCheckBox);
        QWidget::setTabOrder(GridOriginCheckBox, PreviewViewSphereSizeCombo);
        QWidget::setTabOrder(PreviewViewSphereSizeCombo, PreviewViewSphereLocationCombo);
        QWidget::setTabOrder(PreviewViewSphereLocationCombo, PreviewAxisIconCheckBox);
        QWidget::setTabOrder(PreviewAxisIconCheckBox, BackgroundSolidRadio);
        QWidget::setTabOrder(BackgroundSolidRadio, BackgroundSolidColorButton);
        QWidget::setTabOrder(BackgroundSolidColorButton, BackgroundGradientRadio);
        QWidget::setTabOrder(BackgroundGradientRadio, BackgroundGradient1ColorButton);
        QWidget::setTabOrder(BackgroundGradient1ColorButton, BackgroundGradient2ColorButton);
        QWidget::setTabOrder(BackgroundGradient2ColorButton, ObjectSelectedColorButton);
        QWidget::setTabOrder(ObjectSelectedColorButton, ObjectFocusedColorButton);
        QWidget::setTabOrder(ObjectFocusedColorButton, CameraColorButton);
        QWidget::setTabOrder(CameraColorButton, LightColorButton);
        QWidget::setTabOrder(LightColorButton, ControlPointColorButton);
        QWidget::setTabOrder(ControlPointColorButton, ControlPointFocusedColorButton);
        QWidget::setTabOrder(ControlPointFocusedColorButton, ActiveViewColorButton);
        QWidget::setTabOrder(ActiveViewColorButton, InactiveViewColorButton);
        QWidget::setTabOrder(InactiveViewColorButton, TextColorButton);
        QWidget::setTabOrder(TextColorButton, AxesColorButton);
        QWidget::setTabOrder(AxesColorButton, MarqueeBorderColorButton);
        QWidget::setTabOrder(MarqueeBorderColorButton, MarqueeFillColorButton);
        QWidget::setTabOrder(MarqueeFillColorButton, OverlayColorButton);
        QWidget::setTabOrder(OverlayColorButton, gridLineColor);
        QWidget::setTabOrder(gridLineColor, gridStudColor);
        QWidget::setTabOrder(gridStudColor, ViewSphereColorButton);
        QWidget::setTabOrder(ViewSphereColorButton, ViewSphereTextColorButton);
        QWidget::setTabOrder(ViewSphereTextColorButton, ViewSphereHighlightColorButton);
        QWidget::setTabOrder(ViewSphereHighlightColorButton, categoriesTree);
        QWidget::setTabOrder(categoriesTree, partsTree);
        QWidget::setTabOrder(partsTree, importCategories);
        QWidget::setTabOrder(importCategories, exportCategories);
        QWidget::setTabOrder(exportCategories, resetCategories);
        QWidget::setTabOrder(resetCategories, newCategory);
        QWidget::setTabOrder(newCategory, editCategory);
        QWidget::setTabOrder(editCategory, deleteCategory);
        QWidget::setTabOrder(deleteCategory, commandList);
        QWidget::setTabOrder(commandList, KeyboardFilterEdit);
        QWidget::setTabOrder(KeyboardFilterEdit, shortcutsImport);
        QWidget::setTabOrder(shortcutsImport, shortcutsExport);
        QWidget::setTabOrder(shortcutsExport, shortcutsReset);
        QWidget::setTabOrder(shortcutsReset, shortcutEdit);
        QWidget::setTabOrder(shortcutEdit, shortcutAssign);
        QWidget::setTabOrder(shortcutAssign, shortcutRemove);
        QWidget::setTabOrder(shortcutRemove, mouseTree);
        QWidget::setTabOrder(mouseTree, MouseImportButton);
        QWidget::setTabOrder(MouseImportButton, MouseExportButton);
        QWidget::setTabOrder(MouseExportButton, mouseReset);
        QWidget::setTabOrder(mouseReset, mouseButton);
        QWidget::setTabOrder(mouseButton, mouseControl);
        QWidget::setTabOrder(mouseControl, mouseAlt);
        QWidget::setTabOrder(mouseAlt, mouseShift);
        QWidget::setTabOrder(mouseShift, mouseAssign);
        QWidget::setTabOrder(mouseAssign, mouseRemove);
        QWidget::setTabOrder(mouseRemove, mouseSensitivity);

        retranslateUi(lcQPreferencesDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcQPreferencesDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcQPreferencesDialog, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(lcQPreferencesDialog);
    } // setupUi

    void retranslateUi(QDialog *lcQPreferencesDialog)
    {
        lcQPreferencesDialog->setWindowTitle(QCoreApplication::translate("lcQPreferencesDialog", "Preferences", nullptr));
        ApplicationGroup->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Application", nullptr));
        label_19->setText(QCoreApplication::translate("lcQPreferencesDialog", "Language:", nullptr));
        Language->setItemText(0, QCoreApplication::translate("lcQPreferencesDialog", "Default", nullptr));
        Language->setItemText(1, QCoreApplication::translate("lcQPreferencesDialog", "Czech", nullptr));
        Language->setItemText(2, QCoreApplication::translate("lcQPreferencesDialog", "Deutsch ", nullptr));
        Language->setItemText(3, QCoreApplication::translate("lcQPreferencesDialog", "English", nullptr));
        Language->setItemText(4, QCoreApplication::translate("lcQPreferencesDialog", "Fran\303\247ais", nullptr));
        Language->setItemText(5, QCoreApplication::translate("lcQPreferencesDialog", "Portugu\303\252s", nullptr));
        Language->setItemText(6, QCoreApplication::translate("lcQPreferencesDialog", "Espa\303\261ol", nullptr));

        checkForUpdates->setItemText(0, QCoreApplication::translate("lcQPreferencesDialog", "Never", nullptr));
        checkForUpdates->setItemText(1, QCoreApplication::translate("lcQPreferencesDialog", "Once a day", nullptr));
        checkForUpdates->setItemText(2, QCoreApplication::translate("lcQPreferencesDialog", "Once a week", nullptr));

        label_10->setText(QCoreApplication::translate("lcQPreferencesDialog", "Check for updates:", nullptr));
        label_25->setText(QCoreApplication::translate("lcQPreferencesDialog", "Color Theme:", nullptr));
        ColorTheme->setItemText(0, QCoreApplication::translate("lcQPreferencesDialog", "Dark", nullptr));
        ColorTheme->setItemText(1, QCoreApplication::translate("lcQPreferencesDialog", "System", nullptr));

        label_4->setText(QCoreApplication::translate("lcQPreferencesDialog", "Default author name:", nullptr));
        fixedDirectionKeys->setText(QCoreApplication::translate("lcQPreferencesDialog", "Fixed direction keys", nullptr));
        autoLoadMostRecent->setText(QCoreApplication::translate("lcQPreferencesDialog", "Open most recent file on startup", nullptr));
        CustomPathsGroup->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Custom Paths", nullptr));
        label_5->setText(QCoreApplication::translate("lcQPreferencesDialog", "Parts Library:", nullptr));
        label_17->setText(QCoreApplication::translate("lcQPreferencesDialog", "Color Configuration:", nullptr));
        ColorConfigBrowseButton->setText(QCoreApplication::translate("lcQPreferencesDialog", "...", nullptr));
        label_18->setText(QCoreApplication::translate("lcQPreferencesDialog", "Minifig Settings:", nullptr));
        MinifigSettingsBrowseButton->setText(QCoreApplication::translate("lcQPreferencesDialog", "...", nullptr));
        povrayLabel->setText(QCoreApplication::translate("lcQPreferencesDialog", "PO&V-Ray Executable:", nullptr));
        povrayExecutableBrowse->setText(QCoreApplication::translate("lcQPreferencesDialog", "...", nullptr));
        label_9->setText(QCoreApplication::translate("lcQPreferencesDialog", "&LGEO Library:", nullptr));
        lgeoPathBrowse->setText(QCoreApplication::translate("lcQPreferencesDialog", "...", nullptr));
        RestoreTabLayout->setText(QCoreApplication::translate("lcQPreferencesDialog", "Restore tab layout", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabGeneral), QCoreApplication::translate("lcQPreferencesDialog", "General", nullptr));
        MeshLOD->setText(QCoreApplication::translate("lcQPreferencesDialog", "Mesh LOD", nullptr));
        FadeStepsColor->setText(QString());
        ShadingMode->setItemText(0, QCoreApplication::translate("lcQPreferencesDialog", "Wireframe", nullptr));
        ShadingMode->setItemText(1, QCoreApplication::translate("lcQPreferencesDialog", "Flat Shading", nullptr));
        ShadingMode->setItemText(2, QCoreApplication::translate("lcQPreferencesDialog", "Default Lights", nullptr));

        FadeSteps->setText(QCoreApplication::translate("lcQPreferencesDialog", "Fade Previous Steps", nullptr));
        MeshLODLabel->setText(QString());
        antiAliasing->setText(QCoreApplication::translate("lcQPreferencesDialog", "Anti-aliasing", nullptr));
        label_24->setText(QCoreApplication::translate("lcQPreferencesDialog", "Stud Style", nullptr));
        HighlightNewParts->setText(QCoreApplication::translate("lcQPreferencesDialog", "Highlight New Parts", nullptr));
        label_3->setText(QCoreApplication::translate("lcQPreferencesDialog", "Shading Mode:", nullptr));
        studStyleCombo->setItemText(0, QCoreApplication::translate("lcQPreferencesDialog", "Plain", nullptr));
        studStyleCombo->setItemText(1, QCoreApplication::translate("lcQPreferencesDialog", "Thin Lines Logo", nullptr));
        studStyleCombo->setItemText(2, QCoreApplication::translate("lcQPreferencesDialog", "Outline Logo", nullptr));
        studStyleCombo->setItemText(3, QCoreApplication::translate("lcQPreferencesDialog", "Sharp Top Logo", nullptr));
        studStyleCombo->setItemText(4, QCoreApplication::translate("lcQPreferencesDialog", "Rounded Top Logo", nullptr));
        studStyleCombo->setItemText(5, QCoreApplication::translate("lcQPreferencesDialog", "Flattened Logo", nullptr));
        studStyleCombo->setItemText(6, QCoreApplication::translate("lcQPreferencesDialog", "High Contrast", nullptr));
        studStyleCombo->setItemText(7, QCoreApplication::translate("lcQPreferencesDialog", "High Contrast with Logo", nullptr));

        HighContrastButton->setText(QCoreApplication::translate("lcQPreferencesDialog", "Contrast Settings...", nullptr));
        antiAliasingSamples->setItemText(0, QCoreApplication::translate("lcQPreferencesDialog", "2x", nullptr));
        antiAliasingSamples->setItemText(1, QCoreApplication::translate("lcQPreferencesDialog", "4x", nullptr));
        antiAliasingSamples->setItemText(2, QCoreApplication::translate("lcQPreferencesDialog", "8x", nullptr));

        HighlightNewPartsColor->setText(QString());
        label_39->setText(QCoreApplication::translate("lcQPreferencesDialog", "Blender LDraw Render:", nullptr));
        BlenderAddonSettingsButton->setText(QCoreApplication::translate("lcQPreferencesDialog", "Addon Settings...", nullptr));
        groupBox_8->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Lines", nullptr));
        LineWidthLabel->setText(QString());
        ConditionalLinesCheckBox->setText(QCoreApplication::translate("lcQPreferencesDialog", "Conditional Lines", nullptr));
        label_26->setText(QCoreApplication::translate("lcQPreferencesDialog", "Line Width:", nullptr));
        edgeLines->setText(QCoreApplication::translate("lcQPreferencesDialog", "Edge Lines", nullptr));
        AutomateEdgeColor->setText(QCoreApplication::translate("lcQPreferencesDialog", "Automate Edge Color", nullptr));
        AutomateEdgeColorButton->setText(QCoreApplication::translate("lcQPreferencesDialog", "Settings...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabRendering), QCoreApplication::translate("lcQPreferencesDialog", "Rendering", nullptr));
        groupBox_6->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Model Views", nullptr));
        AxisIconCheckBox->setText(QCoreApplication::translate("lcQPreferencesDialog", "Axis Icon", nullptr));
        ViewSphereLocationLabel->setText(QCoreApplication::translate("lcQPreferencesDialog", "Location:", nullptr));
        label_21->setText(QCoreApplication::translate("lcQPreferencesDialog", "View Sphere:", nullptr));
        ViewSphereSizeCombo->setItemText(0, QCoreApplication::translate("lcQPreferencesDialog", "Disabled", nullptr));
        ViewSphereSizeCombo->setItemText(1, QCoreApplication::translate("lcQPreferencesDialog", "Small", nullptr));
        ViewSphereSizeCombo->setItemText(2, QCoreApplication::translate("lcQPreferencesDialog", "Medium", nullptr));
        ViewSphereSizeCombo->setItemText(3, QCoreApplication::translate("lcQPreferencesDialog", "Large", nullptr));

        ViewSphereLocationCombo->setItemText(0, QCoreApplication::translate("lcQPreferencesDialog", "Top Left", nullptr));
        ViewSphereLocationCombo->setItemText(1, QCoreApplication::translate("lcQPreferencesDialog", "Top Right", nullptr));
        ViewSphereLocationCombo->setItemText(2, QCoreApplication::translate("lcQPreferencesDialog", "Bottom Left", nullptr));
        ViewSphereLocationCombo->setItemText(3, QCoreApplication::translate("lcQPreferencesDialog", "Bottom Right", nullptr));

        AxisIconLocationLabel->setText(QCoreApplication::translate("lcQPreferencesDialog", "Location:", nullptr));
        AxisIconLocationCombo->setItemText(0, QCoreApplication::translate("lcQPreferencesDialog", "Top Left", nullptr));
        AxisIconLocationCombo->setItemText(1, QCoreApplication::translate("lcQPreferencesDialog", "Top Right", nullptr));
        AxisIconLocationCombo->setItemText(2, QCoreApplication::translate("lcQPreferencesDialog", "Bottom Left", nullptr));
        AxisIconLocationCombo->setItemText(3, QCoreApplication::translate("lcQPreferencesDialog", "Bottom Right", nullptr));

        groupBox_4->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Base Grid", nullptr));
        label->setText(QCoreApplication::translate("lcQPreferencesDialog", "studs", nullptr));
        gridLines->setText(QCoreApplication::translate("lcQPreferencesDialog", "Draw lines every", nullptr));
        GridOriginCheckBox->setText(QCoreApplication::translate("lcQPreferencesDialog", "Draw origin lines", nullptr));
        gridStuds->setText(QCoreApplication::translate("lcQPreferencesDialog", "Draw studs", nullptr));
        PreviewGroup->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Part Preview", nullptr));
        PreviewAxisIconCheckBox->setText(QCoreApplication::translate("lcQPreferencesDialog", "Axis Icon", nullptr));
        PreviewViewSphereLocationCombo->setItemText(0, QCoreApplication::translate("lcQPreferencesDialog", "Top Left", nullptr));
        PreviewViewSphereLocationCombo->setItemText(1, QCoreApplication::translate("lcQPreferencesDialog", "Top Right", nullptr));
        PreviewViewSphereLocationCombo->setItemText(2, QCoreApplication::translate("lcQPreferencesDialog", "Bottom Left", nullptr));
        PreviewViewSphereLocationCombo->setItemText(3, QCoreApplication::translate("lcQPreferencesDialog", "Bottom Right", nullptr));

        PreviewViewSphereLocationLabel->setText(QCoreApplication::translate("lcQPreferencesDialog", "Location:", nullptr));
        PreviewViewSphereSizeCombo->setItemText(0, QCoreApplication::translate("lcQPreferencesDialog", "Disabled", nullptr));
        PreviewViewSphereSizeCombo->setItemText(1, QCoreApplication::translate("lcQPreferencesDialog", "Small", nullptr));
        PreviewViewSphereSizeCombo->setItemText(2, QCoreApplication::translate("lcQPreferencesDialog", "Medium", nullptr));
        PreviewViewSphereSizeCombo->setItemText(3, QCoreApplication::translate("lcQPreferencesDialog", "Large", nullptr));

        label_8->setText(QCoreApplication::translate("lcQPreferencesDialog", "View Sphere:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("lcQPreferencesDialog", "Views", nullptr));
        groupBox_11->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Interface", nullptr));
        label_15->setText(QCoreApplication::translate("lcQPreferencesDialog", "Marquee Border:", nullptr));
        MarqueeBorderColorButton->setText(QString());
        label_32->setText(QCoreApplication::translate("lcQPreferencesDialog", "Inactive View Border:", nullptr));
        MarqueeFillColorButton->setText(QString());
        label_14->setText(QCoreApplication::translate("lcQPreferencesDialog", "Camera Name:", nullptr));
        AxesColorButton->setText(QString());
        label_16->setText(QCoreApplication::translate("lcQPreferencesDialog", "Marquee Fill:", nullptr));
        InactiveViewColorButton->setText(QString());
        label_2->setText(QCoreApplication::translate("lcQPreferencesDialog", "Overlay Color:", nullptr));
        label_31->setText(QCoreApplication::translate("lcQPreferencesDialog", "Axis Icon Labels:", nullptr));
        TextColorButton->setText(QString());
        OverlayColorButton->setText(QString());
        ActiveViewColorButton->setText(QString());
        label_6->setText(QCoreApplication::translate("lcQPreferencesDialog", "Active View Border:", nullptr));
        groupBox_9->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "View Sphere", nullptr));
        label_22->setText(QCoreApplication::translate("lcQPreferencesDialog", "Color:", nullptr));
        ViewSphereColorButton->setText(QString());
        label_23->setText(QCoreApplication::translate("lcQPreferencesDialog", "Text Color:", nullptr));
        ViewSphereTextColorButton->setText(QString());
        label_20->setText(QCoreApplication::translate("lcQPreferencesDialog", "Highlight Color:", nullptr));
        ViewSphereHighlightColorButton->setText(QString());
        groupBox_10->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Grid", nullptr));
        label_29->setText(QCoreApplication::translate("lcQPreferencesDialog", "Lines:", nullptr));
        gridLineColor->setText(QString());
        label_30->setText(QCoreApplication::translate("lcQPreferencesDialog", "Studs:", nullptr));
        gridStudColor->setText(QString());
        groupBox_7->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Background", nullptr));
        label_27->setText(QCoreApplication::translate("lcQPreferencesDialog", "Top Color:", nullptr));
        BackgroundGradient1ColorButton->setText(QString());
        label_28->setText(QCoreApplication::translate("lcQPreferencesDialog", "Bottom Color:", nullptr));
        BackgroundGradient2ColorButton->setText(QString());
        BackgroundSolidRadio->setText(QCoreApplication::translate("lcQPreferencesDialog", "So&lid Color", nullptr));
        BackgroundGradientRadio->setText(QCoreApplication::translate("lcQPreferencesDialog", "&Gradient", nullptr));
        BackgroundSolidColorButton->setText(QString());
        groupBox_12->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Objects", nullptr));
        label_37->setText(QCoreApplication::translate("lcQPreferencesDialog", "Light:", nullptr));
        LightColorButton->setText(QString());
        label_38->setText(QCoreApplication::translate("lcQPreferencesDialog", "Focused:", nullptr));
        CameraColorButton->setText(QString());
        label_36->setText(QCoreApplication::translate("lcQPreferencesDialog", "Control Point Focused:", nullptr));
        ControlPointColorButton->setText(QString());
        ControlPointFocusedColorButton->setText(QString());
        ObjectSelectedColorButton->setText(QString());
        ObjectFocusedColorButton->setText(QString());
        label_35->setText(QCoreApplication::translate("lcQPreferencesDialog", "Control Point:", nullptr));
        label_33->setText(QCoreApplication::translate("lcQPreferencesDialog", "Selected:", nullptr));
        label_34->setText(QCoreApplication::translate("lcQPreferencesDialog", "Camera:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(ColorTab), QCoreApplication::translate("lcQPreferencesDialog", "Colors", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Parts Library Categories", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = categoriesTree->headerItem();
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("lcQPreferencesDialog", "Categories", nullptr));
        QTreeWidgetItem *___qtreewidgetitem1 = partsTree->headerItem();
        ___qtreewidgetitem1->setText(1, QCoreApplication::translate("lcQPreferencesDialog", "Number", nullptr));
        ___qtreewidgetitem1->setText(0, QCoreApplication::translate("lcQPreferencesDialog", "Name", nullptr));
        importCategories->setText(QCoreApplication::translate("lcQPreferencesDialog", "Import...", nullptr));
        exportCategories->setText(QCoreApplication::translate("lcQPreferencesDialog", "Export...", nullptr));
        resetCategories->setText(QCoreApplication::translate("lcQPreferencesDialog", "Reset...", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Category", nullptr));
        newCategory->setText(QCoreApplication::translate("lcQPreferencesDialog", "New...", nullptr));
        editCategory->setText(QCoreApplication::translate("lcQPreferencesDialog", "Edit...", nullptr));
        deleteCategory->setText(QCoreApplication::translate("lcQPreferencesDialog", "Delete...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabCategories), QCoreApplication::translate("lcQPreferencesDialog", "Categories", nullptr));
        groupBox->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Keyboard Shortcuts", nullptr));
        QTreeWidgetItem *___qtreewidgetitem2 = commandList->headerItem();
        ___qtreewidgetitem2->setText(1, QCoreApplication::translate("lcQPreferencesDialog", "Shortcut", nullptr));
        ___qtreewidgetitem2->setText(0, QCoreApplication::translate("lcQPreferencesDialog", "Command", nullptr));
        KeyboardFilterEdit->setPlaceholderText(QCoreApplication::translate("lcQPreferencesDialog", "Search shortcuts", nullptr));
        shortcutsImport->setText(QCoreApplication::translate("lcQPreferencesDialog", "Import...", nullptr));
        shortcutsExport->setText(QCoreApplication::translate("lcQPreferencesDialog", "Export...", nullptr));
        shortcutsReset->setText(QCoreApplication::translate("lcQPreferencesDialog", "Reset...", nullptr));
        shortcutGroup->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Shortcut", nullptr));
        label_7->setText(QCoreApplication::translate("lcQPreferencesDialog", "Key se&quence:", nullptr));
        shortcutAssign->setText(QCoreApplication::translate("lcQPreferencesDialog", "Assign", nullptr));
        shortcutRemove->setText(QCoreApplication::translate("lcQPreferencesDialog", "Remove", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabKeyboard), QCoreApplication::translate("lcQPreferencesDialog", "Keyboard", nullptr));
        groupBox_5->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Mouse Shortcuts", nullptr));
        QTreeWidgetItem *___qtreewidgetitem3 = mouseTree->headerItem();
        ___qtreewidgetitem3->setText(2, QCoreApplication::translate("lcQPreferencesDialog", "Shortcut 2", nullptr));
        ___qtreewidgetitem3->setText(1, QCoreApplication::translate("lcQPreferencesDialog", "Shortcut 1", nullptr));
        ___qtreewidgetitem3->setText(0, QCoreApplication::translate("lcQPreferencesDialog", "Action", nullptr));
        MouseImportButton->setText(QCoreApplication::translate("lcQPreferencesDialog", "Import...", nullptr));
        MouseExportButton->setText(QCoreApplication::translate("lcQPreferencesDialog", "Export...", nullptr));
        mouseReset->setText(QCoreApplication::translate("lcQPreferencesDialog", "Reset...", nullptr));
        MouseShortcutGroup->setTitle(QCoreApplication::translate("lcQPreferencesDialog", "Shortcut", nullptr));
        label_11->setText(QCoreApplication::translate("lcQPreferencesDialog", "Button:", nullptr));
        mouseButton->setItemText(0, QCoreApplication::translate("lcQPreferencesDialog", "None", nullptr));
        mouseButton->setItemText(1, QCoreApplication::translate("lcQPreferencesDialog", "Left", nullptr));
        mouseButton->setItemText(2, QCoreApplication::translate("lcQPreferencesDialog", "Middle", nullptr));
        mouseButton->setItemText(3, QCoreApplication::translate("lcQPreferencesDialog", "Right", nullptr));

        label_12->setText(QCoreApplication::translate("lcQPreferencesDialog", "Modifiers:", nullptr));
        mouseControl->setText(QCoreApplication::translate("lcQPreferencesDialog", "Control", nullptr));
        mouseAlt->setText(QCoreApplication::translate("lcQPreferencesDialog", "Alt", nullptr));
        mouseShift->setText(QCoreApplication::translate("lcQPreferencesDialog", "Shift", nullptr));
        mouseAssign->setText(QCoreApplication::translate("lcQPreferencesDialog", "Assign", nullptr));
        mouseRemove->setText(QCoreApplication::translate("lcQPreferencesDialog", "Remove", nullptr));
        label_13->setText(QCoreApplication::translate("lcQPreferencesDialog", "Mouse sensitivity:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabMouse), QCoreApplication::translate("lcQPreferencesDialog", "Mouse", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcQPreferencesDialog: public Ui_lcQPreferencesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_QPREFERENCESDIALOG_H
