/********************************************************************************
** Form generated from reading UI file 'lc_aboutdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_ABOUTDIALOG_H
#define UI_LC_ABOUTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_lcAboutDialog
{
public:
    QVBoxLayout *verticalLayout;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QVBoxLayout *verticalLayout_2;
    QLabel *version;
    QLabel *label_4;
    QLabel *label_2;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *info;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcAboutDialog)
    {
        if (lcAboutDialog->objectName().isEmpty())
            lcAboutDialog->setObjectName("lcAboutDialog");
        lcAboutDialog->resize(400, 215);
        verticalLayout = new QVBoxLayout(lcAboutDialog);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(lcAboutDialog);
        label->setObjectName("label");
        label->setPixmap(QPixmap(QString::fromUtf8(":/resources/leocad.png")));
        label->setAlignment(Qt::AlignCenter);
        label->setMargin(16);

        horizontalLayout->addWidget(label);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        version = new QLabel(lcAboutDialog);
        version->setObjectName("version");

        verticalLayout_2->addWidget(version);

        label_4 = new QLabel(lcAboutDialog);
        label_4->setObjectName("label_4");
        label_4->setWordWrap(true);

        verticalLayout_2->addWidget(label_4);

        label_2 = new QLabel(lcAboutDialog);
        label_2->setObjectName("label_2");
        label_2->setText(QString::fromUtf8("<a href=\"https://www.leocad.org\">https://www.leocad.org</a>"));
        label_2->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(label_2);


        horizontalLayout->addLayout(verticalLayout_2);

        horizontalLayout->setStretch(1, 1);

        verticalLayout_3->addLayout(horizontalLayout);

        groupBox = new QGroupBox(lcAboutDialog);
        groupBox->setObjectName("groupBox");
        groupBox->setFlat(false);
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        info = new QLabel(groupBox);
        info->setObjectName("info");
        info->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        gridLayout->addWidget(info, 0, 0, 1, 1);


        verticalLayout_3->addWidget(groupBox);


        verticalLayout->addLayout(verticalLayout_3);

        buttonBox = new QDialogButtonBox(lcAboutDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(lcAboutDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcAboutDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcAboutDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(lcAboutDialog);
    } // setupUi

    void retranslateUi(QDialog *lcAboutDialog)
    {
        lcAboutDialog->setWindowTitle(QCoreApplication::translate("lcAboutDialog", "About LeoCAD", nullptr));
        label->setText(QString());
        version->setText(QCoreApplication::translate("lcAboutDialog", "LeoCAD Version 0.00.0", nullptr));
        label_4->setText(QCoreApplication::translate("lcAboutDialog", "LeoCAD is a free application for creating virtual LEGO models.", nullptr));
        groupBox->setTitle(QCoreApplication::translate("lcAboutDialog", "System Information", nullptr));
        info->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class lcAboutDialog: public Ui_lcAboutDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_ABOUTDIALOG_H
