/********************************************************************************
** Form generated from reading UI file 'lc_qeditgroupsdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_QEDITGROUPSDIALOG_H
#define UI_LC_QEDITGROUPSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_lcQEditGroupsDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTreeWidget *treeWidget;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcQEditGroupsDialog)
    {
        if (lcQEditGroupsDialog->objectName().isEmpty())
            lcQEditGroupsDialog->setObjectName("lcQEditGroupsDialog");
        lcQEditGroupsDialog->resize(450, 350);
        verticalLayout = new QVBoxLayout(lcQEditGroupsDialog);
        verticalLayout->setObjectName("verticalLayout");
        treeWidget = new QTreeWidget(lcQEditGroupsDialog);
        QTreeWidgetItem *__qtreewidgetitem = new QTreeWidgetItem();
        __qtreewidgetitem->setText(0, QString::fromUtf8("1"));
        treeWidget->setHeaderItem(__qtreewidgetitem);
        treeWidget->setObjectName("treeWidget");
        treeWidget->setEditTriggers(QAbstractItemView::NoEditTriggers);
        treeWidget->setDragEnabled(true);
        treeWidget->setDragDropMode(QAbstractItemView::InternalMove);
        treeWidget->setUniformRowHeights(true);
        treeWidget->setHeaderHidden(true);

        verticalLayout->addWidget(treeWidget);

        buttonBox = new QDialogButtonBox(lcQEditGroupsDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(lcQEditGroupsDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcQEditGroupsDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcQEditGroupsDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(lcQEditGroupsDialog);
    } // setupUi

    void retranslateUi(QDialog *lcQEditGroupsDialog)
    {
        lcQEditGroupsDialog->setWindowTitle(QCoreApplication::translate("lcQEditGroupsDialog", "Edit Groups", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcQEditGroupsDialog: public Ui_lcQEditGroupsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_QEDITGROUPSDIALOG_H
