/********************************************************************************
** Form generated from reading UI file 'lc_arraydialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_ARRAYDIALOG_H
#define UI_LC_ARRAYDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_lcArrayDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QGroupBox *groupBox_4;
    QGridLayout *gridLayout_8;
    QGridLayout *gridLayout_9;
    QSpinBox *count3;
    QSpinBox *count1;
    QSpinBox *count2;
    QGroupBox *groupBox_5;
    QGridLayout *gridLayout_10;
    QLineEdit *offset1y;
    QLineEdit *offset2x;
    QLineEdit *offset1x;
    QLineEdit *offset3x;
    QLineEdit *offset1z;
    QLineEdit *offset2y;
    QLineEdit *offset3y;
    QLineEdit *offset2z;
    QLineEdit *offset3z;
    QGroupBox *groupBox_6;
    QGridLayout *gridLayout_11;
    QLineEdit *rotation3z;
    QLineEdit *rotation3x;
    QLineEdit *rotation1x;
    QLineEdit *rotation2z;
    QLineEdit *rotation1z;
    QLineEdit *rotation2x;
    QLineEdit *rotation1y;
    QLineEdit *rotation2y;
    QLineEdit *rotation3y;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcArrayDialog)
    {
        if (lcArrayDialog->objectName().isEmpty())
            lcArrayDialog->setObjectName("lcArrayDialog");
        lcArrayDialog->resize(492, 156);
        verticalLayout = new QVBoxLayout(lcArrayDialog);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        groupBox_4 = new QGroupBox(lcArrayDialog);
        groupBox_4->setObjectName("groupBox_4");
        gridLayout_8 = new QGridLayout(groupBox_4);
        gridLayout_8->setObjectName("gridLayout_8");
        gridLayout_9 = new QGridLayout();
        gridLayout_9->setObjectName("gridLayout_9");
        count3 = new QSpinBox(groupBox_4);
        count3->setObjectName("count3");
        count3->setMinimum(1);
        count3->setMaximum(1000);

        gridLayout_9->addWidget(count3, 2, 0, 1, 1);

        count1 = new QSpinBox(groupBox_4);
        count1->setObjectName("count1");
        count1->setMinimum(1);
        count1->setMaximum(1000);

        gridLayout_9->addWidget(count1, 0, 0, 1, 1);

        count2 = new QSpinBox(groupBox_4);
        count2->setObjectName("count2");
        count2->setMinimum(1);
        count2->setMaximum(1000);

        gridLayout_9->addWidget(count2, 1, 0, 1, 1);


        gridLayout_8->addLayout(gridLayout_9, 0, 0, 1, 1);


        horizontalLayout->addWidget(groupBox_4);

        groupBox_5 = new QGroupBox(lcArrayDialog);
        groupBox_5->setObjectName("groupBox_5");
        gridLayout_10 = new QGridLayout(groupBox_5);
        gridLayout_10->setObjectName("gridLayout_10");
        offset1y = new QLineEdit(groupBox_5);
        offset1y->setObjectName("offset1y");

        gridLayout_10->addWidget(offset1y, 0, 1, 1, 1);

        offset2x = new QLineEdit(groupBox_5);
        offset2x->setObjectName("offset2x");

        gridLayout_10->addWidget(offset2x, 1, 0, 1, 1);

        offset1x = new QLineEdit(groupBox_5);
        offset1x->setObjectName("offset1x");

        gridLayout_10->addWidget(offset1x, 0, 0, 1, 1);

        offset3x = new QLineEdit(groupBox_5);
        offset3x->setObjectName("offset3x");

        gridLayout_10->addWidget(offset3x, 2, 0, 1, 1);

        offset1z = new QLineEdit(groupBox_5);
        offset1z->setObjectName("offset1z");

        gridLayout_10->addWidget(offset1z, 0, 2, 1, 1);

        offset2y = new QLineEdit(groupBox_5);
        offset2y->setObjectName("offset2y");

        gridLayout_10->addWidget(offset2y, 1, 1, 1, 1);

        offset3y = new QLineEdit(groupBox_5);
        offset3y->setObjectName("offset3y");

        gridLayout_10->addWidget(offset3y, 2, 1, 1, 1);

        offset2z = new QLineEdit(groupBox_5);
        offset2z->setObjectName("offset2z");

        gridLayout_10->addWidget(offset2z, 1, 2, 1, 1);

        offset3z = new QLineEdit(groupBox_5);
        offset3z->setObjectName("offset3z");

        gridLayout_10->addWidget(offset3z, 2, 2, 1, 1);


        horizontalLayout->addWidget(groupBox_5);

        groupBox_6 = new QGroupBox(lcArrayDialog);
        groupBox_6->setObjectName("groupBox_6");
        gridLayout_11 = new QGridLayout(groupBox_6);
        gridLayout_11->setObjectName("gridLayout_11");
        rotation3z = new QLineEdit(groupBox_6);
        rotation3z->setObjectName("rotation3z");

        gridLayout_11->addWidget(rotation3z, 2, 2, 1, 1);

        rotation3x = new QLineEdit(groupBox_6);
        rotation3x->setObjectName("rotation3x");

        gridLayout_11->addWidget(rotation3x, 2, 0, 1, 1);

        rotation1x = new QLineEdit(groupBox_6);
        rotation1x->setObjectName("rotation1x");

        gridLayout_11->addWidget(rotation1x, 0, 0, 1, 1);

        rotation2z = new QLineEdit(groupBox_6);
        rotation2z->setObjectName("rotation2z");

        gridLayout_11->addWidget(rotation2z, 1, 2, 1, 1);

        rotation1z = new QLineEdit(groupBox_6);
        rotation1z->setObjectName("rotation1z");

        gridLayout_11->addWidget(rotation1z, 0, 2, 1, 1);

        rotation2x = new QLineEdit(groupBox_6);
        rotation2x->setObjectName("rotation2x");

        gridLayout_11->addWidget(rotation2x, 1, 0, 1, 1);

        rotation1y = new QLineEdit(groupBox_6);
        rotation1y->setObjectName("rotation1y");

        gridLayout_11->addWidget(rotation1y, 0, 1, 1, 1);

        rotation2y = new QLineEdit(groupBox_6);
        rotation2y->setObjectName("rotation2y");

        gridLayout_11->addWidget(rotation2y, 1, 1, 1, 1);

        rotation3y = new QLineEdit(groupBox_6);
        rotation3y->setObjectName("rotation3y");

        gridLayout_11->addWidget(rotation3y, 2, 1, 1, 1);


        horizontalLayout->addWidget(groupBox_6);


        verticalLayout->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(lcArrayDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(count1, offset1x);
        QWidget::setTabOrder(offset1x, offset1y);
        QWidget::setTabOrder(offset1y, offset1z);
        QWidget::setTabOrder(offset1z, rotation1x);
        QWidget::setTabOrder(rotation1x, rotation1y);
        QWidget::setTabOrder(rotation1y, rotation1z);
        QWidget::setTabOrder(rotation1z, count2);
        QWidget::setTabOrder(count2, offset2x);
        QWidget::setTabOrder(offset2x, offset2y);
        QWidget::setTabOrder(offset2y, offset2z);
        QWidget::setTabOrder(offset2z, rotation2x);
        QWidget::setTabOrder(rotation2x, rotation2y);
        QWidget::setTabOrder(rotation2y, rotation2z);
        QWidget::setTabOrder(rotation2z, count3);
        QWidget::setTabOrder(count3, offset3x);
        QWidget::setTabOrder(offset3x, offset3y);
        QWidget::setTabOrder(offset3y, offset3z);
        QWidget::setTabOrder(offset3z, rotation3x);
        QWidget::setTabOrder(rotation3x, rotation3y);
        QWidget::setTabOrder(rotation3y, rotation3z);
        QWidget::setTabOrder(rotation3z, buttonBox);

        retranslateUi(lcArrayDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcArrayDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcArrayDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(lcArrayDialog);
    } // setupUi

    void retranslateUi(QDialog *lcArrayDialog)
    {
        lcArrayDialog->setWindowTitle(QCoreApplication::translate("lcArrayDialog", "Array", nullptr));
        groupBox_4->setTitle(QCoreApplication::translate("lcArrayDialog", "Dimensions", nullptr));
        groupBox_5->setTitle(QCoreApplication::translate("lcArrayDialog", "Offsets", nullptr));
        groupBox_6->setTitle(QCoreApplication::translate("lcArrayDialog", "Rotations", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcArrayDialog: public Ui_lcArrayDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_ARRAYDIALOG_H
