/****************************************************************************
** Meta object code from reading C++ file 'lc_mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../common/lc_mainwindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lc_mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16lcModelTabWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto lcModelTabWidget::qt_create_metaobjectdata<qt_meta_tag_ZN16lcModelTabWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "lcModelTabWidget"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<lcModelTabWidget, qt_meta_tag_ZN16lcModelTabWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject lcModelTabWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16lcModelTabWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16lcModelTabWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16lcModelTabWidgetE_t>.metaTypes,
    nullptr
} };

void lcModelTabWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<lcModelTabWidget *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *lcModelTabWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *lcModelTabWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16lcModelTabWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int lcModelTabWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN12lcMainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto lcMainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN12lcMainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "lcMainWindow",
        "ProjectFileChanged",
        "",
        "Path",
        "PreviewPiece",
        "PartId",
        "ColorCode",
        "ShowPreview",
        "TogglePreviewWidget",
        "Visible",
        "SetCurrentPieceInfo",
        "PieceInfo*",
        "Info",
        "CameraMenuAboutToShow",
        "ProjectionMenuAboutToShow",
        "ViewFocusReceived",
        "UpdateDockWidgetActions",
        "UpdateGamepads",
        "ModelTabContextMenuRequested",
        "QPoint",
        "Point",
        "ModelTabCloseOtherTabs",
        "ModelTabClosed",
        "Index",
        "ModelTabChanged",
        "ClipboardChanged",
        "ActionTriggered",
        "ColorChanged",
        "ColorIndex",
        "PartListPicked",
        "ColorButtonClicked",
        "Print",
        "QPrinter*",
        "Printer",
        "EnableWindowFlags"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'ProjectFileChanged'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'PreviewPiece'
        QtMocHelpers::SlotData<void(const QString &, int, bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { QMetaType::Int, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Slot 'TogglePreviewWidget'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'SetCurrentPieceInfo'
        QtMocHelpers::SlotData<void(PieceInfo *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'CameraMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'ProjectionMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'ViewFocusReceived'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'UpdateDockWidgetActions'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'UpdateGamepads'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'ModelTabContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'ModelTabCloseOtherTabs'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'ModelTabClosed'
        QtMocHelpers::SlotData<void(int)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 23 },
        }}),
        // Slot 'ModelTabChanged'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 23 },
        }}),
        // Slot 'ClipboardChanged'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'ActionTriggered'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'ColorChanged'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 28 },
        }}),
        // Slot 'PartListPicked'
        QtMocHelpers::SlotData<void(PieceInfo *)>(29, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'ColorButtonClicked'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'Print'
        QtMocHelpers::SlotData<void(QPrinter *)>(31, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'EnableWindowFlags'
        QtMocHelpers::SlotData<void(bool)>(34, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<lcMainWindow, qt_meta_tag_ZN12lcMainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject lcMainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12lcMainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12lcMainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12lcMainWindowE_t>.metaTypes,
    nullptr
} };

void lcMainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<lcMainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ProjectFileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->PreviewPiece((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 2: _t->TogglePreviewWidget((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->SetCurrentPieceInfo((*reinterpret_cast<std::add_pointer_t<PieceInfo*>>(_a[1]))); break;
        case 4: _t->CameraMenuAboutToShow(); break;
        case 5: _t->ProjectionMenuAboutToShow(); break;
        case 6: _t->ViewFocusReceived(); break;
        case 7: _t->UpdateDockWidgetActions(); break;
        case 8: _t->UpdateGamepads(); break;
        case 9: _t->ModelTabContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 10: _t->ModelTabCloseOtherTabs(); break;
        case 11: _t->ModelTabClosed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->ModelTabChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->ClipboardChanged(); break;
        case 14: _t->ActionTriggered(); break;
        case 15: _t->ColorChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->PartListPicked((*reinterpret_cast<std::add_pointer_t<PieceInfo*>>(_a[1]))); break;
        case 17: _t->ColorButtonClicked(); break;
        case 18: _t->Print((*reinterpret_cast<std::add_pointer_t<QPrinter*>>(_a[1]))); break;
        case 19: _t->EnableWindowFlags((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *lcMainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *lcMainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12lcMainWindowE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int lcMainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    return _id;
}
QT_WARNING_POP
