/********************************************************************************
** Form generated from reading UI file 'lc_qpropertiesdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_QPROPERTIESDIALOG_H
#define UI_LC_QPROPERTIESDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_lcQPropertiesDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tabSummary;
    QFormLayout *formLayout_3;
    QLabel *label;
    QLineEdit *AuthorEdit;
    QLabel *label_3;
    QLineEdit *DescriptionEdit;
    QLabel *label_2;
    QTextEdit *CommentsEdit;
    QLabel *label_4;
    QLabel *MeasurementsLabel;
    QWidget *tabParts;
    QVBoxLayout *verticalLayout_3;
    QTableWidget *PartsTable;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcQPropertiesDialog)
    {
        if (lcQPropertiesDialog->objectName().isEmpty())
            lcQPropertiesDialog->setObjectName("lcQPropertiesDialog");
        lcQPropertiesDialog->resize(500, 442);
        verticalLayout = new QVBoxLayout(lcQPropertiesDialog);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(lcQPropertiesDialog);
        tabWidget->setObjectName("tabWidget");
        tabSummary = new QWidget();
        tabSummary->setObjectName("tabSummary");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tabSummary->sizePolicy().hasHeightForWidth());
        tabSummary->setSizePolicy(sizePolicy);
        formLayout_3 = new QFormLayout(tabSummary);
        formLayout_3->setObjectName("formLayout_3");
        label = new QLabel(tabSummary);
        label->setObjectName("label");

        formLayout_3->setWidget(0, QFormLayout::ItemRole::LabelRole, label);

        AuthorEdit = new QLineEdit(tabSummary);
        AuthorEdit->setObjectName("AuthorEdit");

        formLayout_3->setWidget(0, QFormLayout::ItemRole::FieldRole, AuthorEdit);

        label_3 = new QLabel(tabSummary);
        label_3->setObjectName("label_3");

        formLayout_3->setWidget(1, QFormLayout::ItemRole::LabelRole, label_3);

        DescriptionEdit = new QLineEdit(tabSummary);
        DescriptionEdit->setObjectName("DescriptionEdit");

        formLayout_3->setWidget(1, QFormLayout::ItemRole::FieldRole, DescriptionEdit);

        label_2 = new QLabel(tabSummary);
        label_2->setObjectName("label_2");

        formLayout_3->setWidget(2, QFormLayout::ItemRole::LabelRole, label_2);

        CommentsEdit = new QTextEdit(tabSummary);
        CommentsEdit->setObjectName("CommentsEdit");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(CommentsEdit->sizePolicy().hasHeightForWidth());
        CommentsEdit->setSizePolicy(sizePolicy1);
        CommentsEdit->setAcceptRichText(false);

        formLayout_3->setWidget(2, QFormLayout::ItemRole::FieldRole, CommentsEdit);

        label_4 = new QLabel(tabSummary);
        label_4->setObjectName("label_4");

        formLayout_3->setWidget(3, QFormLayout::ItemRole::LabelRole, label_4);

        MeasurementsLabel = new QLabel(tabSummary);
        MeasurementsLabel->setObjectName("MeasurementsLabel");

        formLayout_3->setWidget(3, QFormLayout::ItemRole::FieldRole, MeasurementsLabel);

        tabWidget->addTab(tabSummary, QString());
        tabParts = new QWidget();
        tabParts->setObjectName("tabParts");
        verticalLayout_3 = new QVBoxLayout(tabParts);
        verticalLayout_3->setObjectName("verticalLayout_3");
        PartsTable = new QTableWidget(tabParts);
        PartsTable->setObjectName("PartsTable");
        PartsTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
        PartsTable->setSortingEnabled(true);
        PartsTable->horizontalHeader()->setDefaultSectionSize(75);
        PartsTable->horizontalHeader()->setProperty("showSortIndicator", QVariant(true));
        PartsTable->verticalHeader()->setVisible(false);

        verticalLayout_3->addWidget(PartsTable);

        tabWidget->addTab(tabParts, QString());

        verticalLayout->addWidget(tabWidget);

        buttonBox = new QDialogButtonBox(lcQPropertiesDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(tabWidget, buttonBox);
        QWidget::setTabOrder(buttonBox, AuthorEdit);
        QWidget::setTabOrder(AuthorEdit, DescriptionEdit);
        QWidget::setTabOrder(DescriptionEdit, CommentsEdit);
        QWidget::setTabOrder(CommentsEdit, PartsTable);

        retranslateUi(lcQPropertiesDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcQPropertiesDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcQPropertiesDialog, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(lcQPropertiesDialog);
    } // setupUi

    void retranslateUi(QDialog *lcQPropertiesDialog)
    {
        lcQPropertiesDialog->setWindowTitle(QCoreApplication::translate("lcQPropertiesDialog", "Properties", nullptr));
        label->setText(QCoreApplication::translate("lcQPropertiesDialog", "Author:", nullptr));
        label_3->setText(QCoreApplication::translate("lcQPropertiesDialog", "Description:", nullptr));
        label_2->setText(QCoreApplication::translate("lcQPropertiesDialog", "Comments:", nullptr));
        label_4->setText(QCoreApplication::translate("lcQPropertiesDialog", "Measurements:", nullptr));
        MeasurementsLabel->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(tabSummary), QCoreApplication::translate("lcQPropertiesDialog", "Summary", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabParts), QCoreApplication::translate("lcQPropertiesDialog", "Parts Used", nullptr));
    } // retranslateUi

};

namespace Ui {
    class lcQPropertiesDialog: public Ui_lcQPropertiesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_QPROPERTIESDIALOG_H
