/********************************************************************************
** Form generated from reading UI file 'lc_qupdatedialog.ui'
**
** Created by: Qt User Interface Compiler version 6.10.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LC_QUPDATEDIALOG_H
#define UI_LC_QUPDATEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_lcQUpdateDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *status;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *lcQUpdateDialog)
    {
        if (lcQUpdateDialog->objectName().isEmpty())
            lcQUpdateDialog->setObjectName("lcQUpdateDialog");
        lcQUpdateDialog->resize(400, 98);
        verticalLayout = new QVBoxLayout(lcQUpdateDialog);
        verticalLayout->setObjectName("verticalLayout");
        status = new QLabel(lcQUpdateDialog);
        status->setObjectName("status");
        status->setTextFormat(Qt::RichText);
        status->setOpenExternalLinks(true);

        verticalLayout->addWidget(status);

        buttonBox = new QDialogButtonBox(lcQUpdateDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(lcQUpdateDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, lcQUpdateDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, lcQUpdateDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(lcQUpdateDialog);
    } // setupUi

    void retranslateUi(QDialog *lcQUpdateDialog)
    {
        lcQUpdateDialog->setWindowTitle(QCoreApplication::translate("lcQUpdateDialog", "LeoCAD Updates", nullptr));
        status->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class lcQUpdateDialog: public Ui_lcQUpdateDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LC_QUPDATEDIALOG_H
